@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2013-2014 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2013-2016 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -23,6 +23,7 @@
 #define __GS_APP_H
 
 #include <glib-object.h>
+#include <gdk/gdk.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <appstream-glib.h>
 
@@ -34,11 +35,25 @@ G_BEGIN_DECLS
 
 G_DECLARE_FINAL_TYPE (GsApp, gs_app, GS, APP, GObject)
 
-typedef enum {
-	GS_APP_ERROR_FAILED,
-	GS_APP_ERROR_LAST
-} GsAppError;
-
+/**
+ * GsAppKudo:
+ * @GS_APP_KUDO_MY_LANGUAGE:		Localised in my language
+ * @GS_APP_KUDO_RECENT_RELEASE:		Released recently
+ * @GS_APP_KUDO_FEATURED_RECOMMENDED:	Chosen for the front page
+ * @GS_APP_KUDO_MODERN_TOOLKIT:		Uses a modern toolkit
+ * @GS_APP_KUDO_SEARCH_PROVIDER:	Provides a search provider
+ * @GS_APP_KUDO_INSTALLS_USER_DOCS:	Installs user docs
+ * @GS_APP_KUDO_USES_NOTIFICATIONS:	Registers notifications
+ * @GS_APP_KUDO_HAS_KEYWORDS:		Has at least 1 keyword
+ * @GS_APP_KUDO_USES_APP_MENU:		Uses an AppMenu for navigation
+ * @GS_APP_KUDO_HAS_SCREENSHOTS:	Supplies screenshots
+ * @GS_APP_KUDO_POPULAR:		Is popular
+ * @GS_APP_KUDO_PERFECT_SCREENSHOTS:	Supplies perfect screenshots
+ * @GS_APP_KUDO_HIGH_CONTRAST:		Installs a high contrast icon
+ * @GS_APP_KUDO_HI_DPI_ICON:		Installs a HiDPI icon
+ *
+ * Any awards given to the application.
+ **/
 typedef enum {
 	GS_APP_KUDO_MY_LANGUAGE			= 1 << 0,
 	GS_APP_KUDO_RECENT_RELEASE		= 1 << 1,
@@ -51,10 +66,10 @@ typedef enum {
 	GS_APP_KUDO_USES_APP_MENU		= 1 << 8,
 	GS_APP_KUDO_HAS_SCREENSHOTS		= 1 << 9,
 	GS_APP_KUDO_POPULAR			= 1 << 10,
-	GS_APP_KUDO_IBUS_HAS_SYMBOL		= 1 << 11,
 	GS_APP_KUDO_PERFECT_SCREENSHOTS		= 1 << 12,
 	GS_APP_KUDO_HIGH_CONTRAST		= 1 << 13,
 	GS_APP_KUDO_HI_DPI_ICON			= 1 << 14,
+	/*< private >*/
 	GS_APP_KUDO_LAST
 } GsAppKudo;
 
@@ -62,31 +77,24 @@ typedef enum {
 #define	GS_APP_INSTALL_DATE_UNKNOWN		1 /* 1s past the epoch */
 #define	GS_APP_SIZE_UNKNOWABLE			G_MAXUINT64
 
+/**
+ * GsAppQuality:
+ * @GS_APP_QUALITY_UNKNOWN:	The quality value is unknown
+ * @GS_APP_QUALITY_LOWEST:	Lowest quality
+ * @GS_APP_QUALITY_NORMAL:	Normal quality
+ * @GS_APP_QUALITY_HIGHEST:	Highest quality
+ *
+ * Any awards given to the application.
+ **/
 typedef enum {
 	GS_APP_QUALITY_UNKNOWN,
 	GS_APP_QUALITY_LOWEST,
 	GS_APP_QUALITY_NORMAL,
 	GS_APP_QUALITY_HIGHEST,
+	/*< private >*/
 	GS_APP_QUALITY_LAST
 } GsAppQuality;
 
-#define	GS_APP_KUDOS_WEIGHT_TO_PERCENTAGE(w)	(w * 20)
-
-#if !AS_CHECK_VERSION(0,5,12)
-#define AS_APP_QUIRK_MATCH_ANY_PREFIX		(1 << 3)
-#endif
-
-#if !AS_CHECK_VERSION(0,5,14)
-#define AS_APP_QUIRK_NEEDS_REBOOT		(1 << 4)
-#define AS_APP_QUIRK_NOT_REVIEWABLE		(1 << 5)
-#endif
-
-#if !AS_CHECK_VERSION(0,5,15)
-#define AS_APP_QUIRK_HAS_SHORTCUT		(1 << 6)
-#endif
-
-GQuark		 gs_app_error_quark		(void);
-
 GsApp		*gs_app_new			(const gchar	*id);
 gchar		*gs_app_to_string		(GsApp		*app);
 
@@ -123,7 +131,7 @@ void		 gs_app_set_source_ids		(GsApp		*app,
 void		 gs_app_clear_source_ids	(GsApp		*app);
 const gchar	*gs_app_get_project_group	(GsApp		*app);
 void		 gs_app_set_project_group	(GsApp		*app,
-						 const gchar	*source);
+						 const gchar	*project_group);
 const gchar	*gs_app_get_version		(GsApp		*app);
 const gchar	*gs_app_get_version_ui		(GsApp		*app);
 void		 gs_app_set_version		(GsApp		*app,
@@ -134,7 +142,7 @@ void		 gs_app_set_summary		(GsApp		*app,
 						 const gchar	*summary);
 const gchar	*gs_app_get_summary_missing	(GsApp		*app);
 void		 gs_app_set_summary_missing	(GsApp		*app,
-						 const gchar	*missing);
+						 const gchar	*summary_missing);
 const gchar	*gs_app_get_description		(GsApp		*app);
 void		 gs_app_set_description		(GsApp		*app,
 						 GsAppQuality	 quality,
@@ -182,13 +190,10 @@ void		 gs_app_set_icon		(GsApp		*app,
 						 AsIcon		*icon);
 GFile		*gs_app_get_local_file		(GsApp		*app);
 void		 gs_app_set_local_file		(GsApp		*app,
-						 GFile		*icon);
+						 GFile		*local_file);
 GsApp		*gs_app_get_runtime		(GsApp		*app);
 void		 gs_app_set_runtime		(GsApp		*app,
 						 GsApp		*runtime);
-gboolean	 gs_app_load_icon		(GsApp		*app,
-						 gint		 scale,
-						 GError		**error);
 const gchar	*gs_app_get_metadata_item	(GsApp		*app,
 						 const gchar	*key);
 void		 gs_app_set_metadata		(GsApp		*app,
@@ -226,6 +231,11 @@ void		 gs_app_set_install_date	(GsApp		*app,
 GPtrArray	*gs_app_get_categories		(GsApp		*app);
 void		 gs_app_set_categories		(GsApp		*app,
 						 GPtrArray	*categories);
+GPtrArray	*gs_app_get_key_colors		(GsApp		*app);
+void		 gs_app_set_key_colors		(GsApp		*app,
+						 GPtrArray	*key_colors);
+void		 gs_app_add_key_color		(GsApp		*app,
+						 GdkRGBA	*key_color);
 gboolean	 gs_app_has_category		(GsApp		*app,
 						 const gchar	*category);
 void		 gs_app_add_category		(GsApp		*app,
@@ -250,9 +260,6 @@ void		 gs_app_add_quirk		(GsApp		*app,
 						 AsAppQuirk	 quirk);
 void		 gs_app_remove_quirk		(GsApp		*app,
 						 AsAppQuirk	 quirk);
-GError		*gs_app_get_last_error		(GsApp		*app);
-void		 gs_app_set_last_error		(GsApp		*app,
-						 GError		*error);
 G_END_DECLS
 
 #endif /* __GS_APP_H */
