@@ -99,7 +99,9 @@ gs_application_init (GsApplication *application)
 		{ "search", '\0', 0, G_OPTION_ARG_STRING, NULL,
 		  _("Search for applications"), _("SEARCH") },
 		{ "details", '\0', 0, G_OPTION_ARG_STRING, NULL,
-		  _("Show application details"), _("ID") },
+		  _("Show application details (using application ID)"), _("ID") },
+		{ "details-pkg", '\0', 0, G_OPTION_ARG_STRING, NULL,
+		  _("Show application details (using package name)"), _("PKGNAME") },
 		{ "local-filename", '\0', 0, G_OPTION_ARG_FILENAME, NULL,
 		  _("Open a local package file"), _("FILENAME") },
 		{ "verbose", '\0', 0, G_OPTION_ARG_NONE, NULL,
@@ -116,6 +118,7 @@ gs_application_init (GsApplication *application)
 	g_application_add_main_option_entries (G_APPLICATION (application), options);
 
 	application->profile = as_profile_new ();
+	application->network_changed_handler = 0;
 }
 
 static void
@@ -179,7 +182,7 @@ gs_application_monitor_network (GsApplication *app)
 	GNetworkMonitor *network_monitor;
 
 	network_monitor = g_network_monitor_get_default ();
-	if (network_monitor == NULL)
+	if (network_monitor == NULL || app->network_changed_handler != 0)
 		return;
 	app->network_monitor = g_object_ref (network_monitor);
 
@@ -590,8 +593,26 @@ details_activated (GSimpleAction *action,
 	g_variant_get (parameter, "(&s&s)", &id, &search);
 	if (search != NULL && search[0] != '\0')
 		gs_shell_show_search_result (app->shell, id, search);
-	else
-		gs_shell_show_details (app->shell, id);
+	else {
+		g_autoptr (GsApp) a = NULL;
+		a = gs_app_new (id);
+		gs_shell_show_app (app->shell, a);
+	}
+}
+
+static void
+details_pkg_activated (GSimpleAction *action,
+		       GVariant      *parameter,
+		       gpointer       data)
+{
+	GsApplication *app = GS_APPLICATION (data);
+	g_autoptr (GsApp) a = NULL;
+
+	initialize_ui_and_present_window (app);
+
+	a = gs_app_new (NULL);
+	gs_app_add_source (a, g_variant_get_string (parameter, NULL));
+	gs_shell_show_app (app->shell, a);
 }
 
 static void
@@ -621,7 +642,7 @@ launch_activated (GSimpleAction *action,
 
 	desktop_id = g_variant_get_string (parameter, NULL);
 	display = gdk_display_get_default ();
-	appinfo = G_APP_INFO (g_desktop_app_info_new (desktop_id));
+	appinfo = G_APP_INFO (gs_utils_get_desktop_app_info (desktop_id));
 	if (appinfo == NULL) {
 		g_warning ("no such desktop file: %s", desktop_id);
 		return;
@@ -686,6 +707,7 @@ static GActionEntry actions[] = {
 	{ "set-mode", set_mode_activated, "s", NULL, NULL },
 	{ "search", search_activated, "s", NULL, NULL },
 	{ "details", details_activated, "(ss)", NULL, NULL },
+	{ "details-pkg", details_pkg_activated, "s", NULL, NULL },
 	{ "filename", filename_activated, "(s)", NULL, NULL },
 	{ "launch", launch_activated, "s", NULL, NULL },
 	{ "show-offline-update-error", show_offline_updates_error, NULL, NULL, NULL },
@@ -737,7 +759,6 @@ gs_application_startup (GApplication *application)
 
 	gs_application_monitor_permission (GS_APPLICATION (application));
 	gs_application_monitor_updates (GS_APPLICATION (application));
-	gs_application_monitor_network (GS_APPLICATION (application));
 	gs_folders_convert ();
 
 	gs_application_update_software_sources_presence (application);
@@ -755,6 +776,7 @@ gs_application_activate (GApplication *application)
 	GsApplication *app = GS_APPLICATION (application);
 
 	gs_application_initialize_ui (GS_APPLICATION (application));
+	gs_application_monitor_network (GS_APPLICATION (application));
 
 	/* start metadata loading screen */
 	if (gs_shell_get_mode (app->shell) == GS_SHELL_MODE_UNKNOWN) {
@@ -803,6 +825,7 @@ static int
 gs_application_handle_local_options (GApplication *app, GVariantDict *options)
 {
 	const gchar *id;
+	const gchar *pkgname;
 	const gchar *local_filename;
 	const gchar *mode;
 	const gchar *search;
@@ -847,6 +870,11 @@ gs_application_handle_local_options (GApplication *app, GVariantDict *options)
 						"details",
 						g_variant_new ("(ss)", id, ""));
 		return 0;
+	} else if (g_variant_dict_lookup (options, "details-pkg", "&s", &pkgname)) {
+		g_action_group_activate_action (G_ACTION_GROUP (app),
+						"details-pkg",
+						g_variant_new_string (pkgname));
+		return 0;
 	} else if (g_variant_dict_lookup (options, "local-filename", "^&ay", &local_filename)) {
 		g_action_group_activate_action (G_ACTION_GROUP (app),
 						"filename",
@@ -885,6 +913,17 @@ gs_application_open (GApplication  *application,
 			                                "details",
 			                                g_variant_new ("(ss)", path, ""));
 		}
+		if (g_strcmp0 (soup_uri_get_scheme (uri), "apt") == 0) {
+			const gchar *path = soup_uri_get_path (uri);
+
+			/* trim any leading slashes */
+			while (*path == '/')
+				path++;
+
+			g_action_group_activate_action (G_ACTION_GROUP (app),
+			                                "details-pkg",
+			                                g_variant_new_string (path));
+		}
 	}
 }
 
