@@ -26,7 +26,7 @@
 #include <gtk/gtk.h>
 
 #include "gs-history-dialog.h"
-#include "gs-utils.h"
+#include "gs-common.h"
 
 struct _GsHistoryDialog
 {
@@ -89,6 +89,7 @@ gs_history_dialog_set_app (GsHistoryDialog *dialog, GsApp *app)
 			tmp = C_("app status", "Installed");
 			break;
 		case AS_APP_STATE_UPDATABLE:
+		case AS_APP_STATE_UPDATABLE_LIVE:
 			/* TRANSLATORS: this is the status in the history UI,
 			 * where we are showing the application was updated */
 			tmp = C_("app status", "Updated");
@@ -138,6 +139,8 @@ gs_history_dialog_set_app (GsHistoryDialog *dialog, GsApp *app)
 			      "margin-top", 6,
 			      "margin-bottom", 6,
 			      "xalign", 1.0,
+			      "ellipsize", PANGO_ELLIPSIZE_END,
+			      "width-chars", 10,
 			      NULL);
 		gtk_size_group_add_widget (dialog->sizegroup_version, widget);
 		gtk_box_pack_start (box, widget, TRUE, TRUE, 0);
