@@ -24,10 +24,10 @@
 #include <string.h>
 #include <glib/gi18n.h>
 
-#include "gs-app.h"
+#include "gs-app-private.h"
 #include "gs-page.h"
 #include "gs-shell.h"
-#include "gs-utils.h"
+#include "gs-common.h"
 
 typedef struct
 {
