@@ -31,7 +31,6 @@
 G_BEGIN_DECLS
 
 #define GS_TYPE_PLUGIN_LOADER		(gs_plugin_loader_get_type ())
-#define GS_PLUGIN_LOADER_ERROR		(gs_plugin_loader_error_quark ())
 
 G_DECLARE_DERIVABLE_TYPE (GsPluginLoader, gs_plugin_loader, GS, PLUGIN_LOADER, GObject)
 
@@ -45,13 +44,6 @@ struct _GsPluginLoaderClass
 	void			(*updates_changed)	(GsPluginLoader	*plugin_loader);
 };
 
-typedef enum
-{
-	GS_PLUGIN_LOADER_ERROR_FAILED,
-	GS_PLUGIN_LOADER_ERROR_NO_RESULTS,
-	GS_PLUGIN_LOADER_ERROR_LAST
-} GsPluginLoaderError;
-
 typedef enum {
 	GS_PLUGIN_LOADER_ACTION_INSTALL,
 	GS_PLUGIN_LOADER_ACTION_REMOVE,
@@ -63,6 +55,7 @@ typedef enum {
 	GS_PLUGIN_LOADER_ACTION_UPDATE_CANCEL,
 	GS_PLUGIN_LOADER_ACTION_ADD_SHORTCUT,
 	GS_PLUGIN_LOADER_ACTION_REMOVE_SHORTCUT,
+	GS_PLUGIN_LOADER_ACTION_ADD_SOURCE,
 	GS_PLUGIN_LOADER_ACTION_LAST
 } GsPluginLoaderAction;
 
@@ -70,15 +63,13 @@ typedef void	 (*GsPluginLoaderFinishedFunc)		(GsPluginLoader	*plugin_loader,
 							 GsApp		*app,
 							 gpointer	 user_data);
 
-GQuark		 gs_plugin_loader_error_quark		(void);
-
 GsPluginLoader	*gs_plugin_loader_new			(void);
 void		 gs_plugin_loader_get_installed_async	(GsPluginLoader	*plugin_loader,
 							 GsPluginRefineFlags flags,
 							 GCancellable	*cancellable,
 							 GAsyncReadyCallback callback,
 							 gpointer	 user_data);
-GList		*gs_plugin_loader_get_installed_finish	(GsPluginLoader	*plugin_loader,
+GsAppList	*gs_plugin_loader_get_installed_finish	(GsPluginLoader	*plugin_loader,
 							 GAsyncResult	*res,
 							 GError		**error);
 void		 gs_plugin_loader_get_updates_async	(GsPluginLoader	*plugin_loader,
@@ -86,7 +77,7 @@ void		 gs_plugin_loader_get_updates_async	(GsPluginLoader	*plugin_loader,
 							 GCancellable	*cancellable,
 							 GAsyncReadyCallback callback,
 							 gpointer	 user_data);
-GList		*gs_plugin_loader_get_updates_finish	(GsPluginLoader	*plugin_loader,
+GsAppList	*gs_plugin_loader_get_updates_finish	(GsPluginLoader	*plugin_loader,
 							 GAsyncResult	*res,
 							 GError		**error);
 void		 gs_plugin_loader_get_distro_upgrades_async (GsPluginLoader	*plugin_loader,
@@ -94,7 +85,7 @@ void		 gs_plugin_loader_get_distro_upgrades_async (GsPluginLoader	*plugin_loader
 							 GCancellable	*cancellable,
 							 GAsyncReadyCallback callback,
 							 gpointer	 user_data);
-GList		*gs_plugin_loader_get_distro_upgrades_finish (GsPluginLoader	*plugin_loader,
+GsAppList	*gs_plugin_loader_get_distro_upgrades_finish (GsPluginLoader	*plugin_loader,
 							 GAsyncResult	*res,
 							 GError		**error);
 void		 gs_plugin_loader_get_unvoted_reviews_async (GsPluginLoader	*plugin_loader,
@@ -102,7 +93,7 @@ void		 gs_plugin_loader_get_unvoted_reviews_async (GsPluginLoader	*plugin_loader
 							 GCancellable	*cancellable,
 							 GAsyncReadyCallback callback,
 							 gpointer	 user_data);
-GList		*gs_plugin_loader_get_unvoted_reviews_finish (GsPluginLoader	*plugin_loader,
+GsAppList	*gs_plugin_loader_get_unvoted_reviews_finish (GsPluginLoader	*plugin_loader,
 							 GAsyncResult	*res,
 							 GError		**error);
 void		 gs_plugin_loader_get_sources_async	(GsPluginLoader	*plugin_loader,
@@ -110,7 +101,7 @@ void		 gs_plugin_loader_get_sources_async	(GsPluginLoader	*plugin_loader,
 							 GCancellable	*cancellable,
 							 GAsyncReadyCallback callback,
 							 gpointer	 user_data);
-GList		*gs_plugin_loader_get_sources_finish	(GsPluginLoader	*plugin_loader,
+GsAppList	*gs_plugin_loader_get_sources_finish	(GsPluginLoader	*plugin_loader,
 							 GAsyncResult	*res,
 							 GError		**error);
 void		 gs_plugin_loader_get_popular_async	(GsPluginLoader	*plugin_loader,
@@ -118,7 +109,7 @@ void		 gs_plugin_loader_get_popular_async	(GsPluginLoader	*plugin_loader,
 							 GCancellable	*cancellable,
 							 GAsyncReadyCallback callback,
 							 gpointer	 user_data);
-GList		*gs_plugin_loader_get_popular_finish	(GsPluginLoader	*plugin_loader,
+GsAppList	*gs_plugin_loader_get_popular_finish	(GsPluginLoader	*plugin_loader,
 							 GAsyncResult	*res,
 							 GError		**error);
 void		 gs_plugin_loader_get_featured_async	(GsPluginLoader	*plugin_loader,
@@ -126,7 +117,7 @@ void		 gs_plugin_loader_get_featured_async	(GsPluginLoader	*plugin_loader,
 							 GCancellable	*cancellable,
 							 GAsyncReadyCallback callback,
 							 gpointer	 user_data);
-GList		*gs_plugin_loader_get_featured_finish	(GsPluginLoader	*plugin_loader,
+GsAppList	*gs_plugin_loader_get_featured_finish	(GsPluginLoader	*plugin_loader,
 							 GAsyncResult	*res,
 							 GError		**error);
 void		 gs_plugin_loader_get_categories_async	(GsPluginLoader	*plugin_loader,
@@ -134,7 +125,7 @@ void		 gs_plugin_loader_get_categories_async	(GsPluginLoader	*plugin_loader,
 							 GCancellable	*cancellable,
 							 GAsyncReadyCallback callback,
 							 gpointer	 user_data);
-GList		*gs_plugin_loader_get_categories_finish	(GsPluginLoader	*plugin_loader,
+GPtrArray	*gs_plugin_loader_get_categories_finish	(GsPluginLoader	*plugin_loader,
 							 GAsyncResult	*res,
 							 GError		**error);
 void		 gs_plugin_loader_get_category_apps_async (GsPluginLoader	*plugin_loader,
@@ -143,7 +134,7 @@ void		 gs_plugin_loader_get_category_apps_async (GsPluginLoader	*plugin_loader,
 							 GCancellable	*cancellable,
 							 GAsyncReadyCallback callback,
 							 gpointer	 user_data);
-GList		*gs_plugin_loader_get_category_apps_finish (GsPluginLoader	*plugin_loader,
+GsAppList	*gs_plugin_loader_get_category_apps_finish (GsPluginLoader	*plugin_loader,
 							 GAsyncResult	*res,
 							 GError		**error);
 void		 gs_plugin_loader_search_async		(GsPluginLoader	*plugin_loader,
@@ -152,7 +143,7 @@ void		 gs_plugin_loader_search_async		(GsPluginLoader	*plugin_loader,
 							 GCancellable	*cancellable,
 							 GAsyncReadyCallback callback,
 							 gpointer	 user_data);
-GList		*gs_plugin_loader_search_finish		(GsPluginLoader	*plugin_loader,
+GsAppList	*gs_plugin_loader_search_finish		(GsPluginLoader	*plugin_loader,
 							 GAsyncResult	*res,
 							 GError		**error);
 void		 gs_plugin_loader_search_files_async	(GsPluginLoader	*plugin_loader,
@@ -161,7 +152,7 @@ void		 gs_plugin_loader_search_files_async	(GsPluginLoader	*plugin_loader,
 							 GCancellable	*cancellable,
 							 GAsyncReadyCallback callback,
 							 gpointer	 user_data);
-GList		*gs_plugin_loader_search_files_finish	(GsPluginLoader	*plugin_loader,
+GsAppList	*gs_plugin_loader_search_files_finish	(GsPluginLoader	*plugin_loader,
 							 GAsyncResult	*res,
 							 GError		**error);
 void		 gs_plugin_loader_search_what_provides_async (GsPluginLoader	*plugin_loader,
@@ -170,7 +161,7 @@ void		 gs_plugin_loader_search_what_provides_async (GsPluginLoader	*plugin_loade
 							 GCancellable	*cancellable,
 							 GAsyncReadyCallback callback,
 							 gpointer	 user_data);
-GList		*gs_plugin_loader_search_what_provides_finish (GsPluginLoader	*plugin_loader,
+GsAppList	*gs_plugin_loader_search_what_provides_finish (GsPluginLoader	*plugin_loader,
 							 GAsyncResult	*res,
 							 GError		**error);
 void		 gs_plugin_loader_file_to_app_async	(GsPluginLoader	*plugin_loader,
@@ -183,7 +174,7 @@ GsApp		*gs_plugin_loader_file_to_app_finish	(GsPluginLoader	*plugin_loader,
 							 GAsyncResult	*res,
 							 GError		**error);
 void		 gs_plugin_loader_update_async		(GsPluginLoader	*plugin_loader,
-							 GList		*apps,
+							 GsAppList	*apps,
 							 GCancellable	*cancellable,
 							 GAsyncReadyCallback callback,
 							 gpointer	 user_data);
@@ -226,9 +217,6 @@ void		 gs_plugin_loader_review_action_async	(GsPluginLoader	*plugin_loader,
 							 GCancellable	*cancellable,
 							 GAsyncReadyCallback callback,
 							 gpointer	 user_data);
-gboolean	 gs_plugin_loader_review_action_finish	(GsPluginLoader	*plugin_loader,
-							 GAsyncResult	*res,
-							 GError		**error);
 gboolean	 gs_plugin_loader_refresh_finish	(GsPluginLoader	*plugin_loader,
 							 GAsyncResult	*res,
 							 GError		**error);
@@ -238,7 +226,7 @@ void		 gs_plugin_loader_refresh_async		(GsPluginLoader	*plugin_loader,
 							 GCancellable	*cancellable,
 							 GAsyncReadyCallback callback,
 							 gpointer	 user_data);
-GPtrArray	*gs_plugin_loader_get_pending		(GsPluginLoader	*plugin_loader);
+GsAppList	*gs_plugin_loader_get_pending		(GsPluginLoader	*plugin_loader);
 void		 gs_plugin_loader_set_network_status    (GsPluginLoader *plugin_loader,
 							 gboolean        online);
 gboolean	 gs_plugin_loader_get_plugin_supported	(GsPluginLoader	*plugin_loader,
