@@ -19,7 +19,12 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
-/* Introduction:
+/**
+ * SECTION:gs-plugin
+ * @title: GsPlugin
+ * @include: gnome-software.h
+ * @stability: Unstable
+ * @short_description: Runtime-loaded modules providing functionality
  *
  * Plugins are modules that are loaded at runtime to provide information
  * about requests and to service user actions like installing, removing
@@ -45,6 +50,7 @@
 #include "gs-os-release.h"
 #include "gs-plugin-private.h"
 #include "gs-plugin.h"
+#include "gs-utils.h"
 
 typedef struct
 {
@@ -55,9 +61,7 @@ typedef struct
 	GsPluginData		*data;			/* for gs-plugin-{name}.c */
 	GsPluginFlags		 flags;
 	SoupSession		*soup_session;
-	const gchar		**conflicts;		/* allow-none */
-	const gchar		**order_after;		/* allow-none */
-	const gchar		**order_before;		/* allow-none */
+	GPtrArray		*rules[GS_PLUGIN_RULE_LAST];
 	gboolean		 enabled;
 	gchar			*locale;		/* allow-none */
 	gchar			*name;
@@ -86,7 +90,12 @@ typedef const gchar	**(*GsPluginGetDepsFunc)	(GsPlugin	*plugin);
 
 /**
  * gs_plugin_status_to_string:
- */
+ * @status: a #GsPluginStatus, e.g. %GS_PLUGIN_STATUS_DOWNLOADING
+ *
+ * Converts the #GsPluginStatus enum to a string.
+ *
+ * Returns: the string representation, or "unknown"
+ **/
 const gchar *
 gs_plugin_status_to_string (GsPluginStatus status)
 {
@@ -108,14 +117,17 @@ gs_plugin_status_to_string (GsPluginStatus status)
 }
 /**
  * gs_plugin_create:
+ * @filename: an absolute filename
+ * @error: a #GError, or %NULL
+ *
+ * Creates a new plugin from an external module.
+ *
+ * Returns: the #GsPlugin or %NULL
  **/
 GsPlugin *
 gs_plugin_create (const gchar *filename, GError **error)
 {
 	GModule *module;
-	GsPluginGetDepsFunc order_after = NULL;
-	GsPluginGetDepsFunc order_before = NULL;
-	GsPluginGetDepsFunc plugin_conflicts = NULL;
 	GsPlugin *plugin = NULL;
 	GsPluginPrivate *priv;
 	g_autofree gchar *basename = NULL;
@@ -142,24 +154,10 @@ gs_plugin_create (const gchar *filename, GError **error)
 	}
 	g_strdelimit (basename, ".", '\0');
 
-	/* get plugins this plugin depends on */
-	g_module_symbol (module,
-			 "gs_plugin_order_after",
-			 (gpointer *) &order_after);
-	g_module_symbol (module,
-			 "gs_plugin_order_before",
-			 (gpointer *) &order_before);
-	g_module_symbol (module,
-			 "gs_plugin_get_conflicts",
-			 (gpointer *) &plugin_conflicts);
-
 	/* create new plugin */
 	plugin = gs_plugin_new ();
 	priv = gs_plugin_get_instance_private (plugin);
 	priv->module = module;
-	priv->order_after = order_after != NULL ? order_after (plugin) : NULL;
-	priv->order_before = order_before != NULL ? order_before (plugin) : NULL;
-	priv->conflicts = plugin_conflicts != NULL ? plugin_conflicts (plugin) : NULL;
 	priv->name = g_strdup (basename + 13);
 	return plugin;
 }
@@ -172,6 +170,10 @@ gs_plugin_finalize (GObject *object)
 {
 	GsPlugin *plugin = GS_PLUGIN (object);
 	GsPluginPrivate *priv = gs_plugin_get_instance_private (plugin);
+	guint i;
+
+	for (i = 0; i < GS_PLUGIN_RULE_LAST; i++)
+		g_ptr_array_unref (priv->rules[i]);
 
 	if (priv->timer_id > 0)
 		g_source_remove (priv->timer_id);
@@ -187,6 +189,12 @@ gs_plugin_finalize (GObject *object)
 
 /**
  * gs_plugin_get_data:
+ * @plugin: a #GsPlugin
+ *
+ * Gets the private data for the plugin if gs_plugin_alloc_data() has
+ * been called.
+ *
+ * Returns: the #GsPluginData, or %NULL
  **/
 GsPluginData *
 gs_plugin_get_data (GsPlugin *plugin)
@@ -198,6 +206,15 @@ gs_plugin_get_data (GsPlugin *plugin)
 
 /**
  * gs_plugin_alloc_data:
+ * @plugin: a #GsPlugin
+ * @sz: the size of data to allocate, e.g. `sizeof(FooPluginPrivate)`
+ *
+ * Allocates a private data area for the plugin which can be retrieved
+ * using gs_plugin_get_data().
+ * This is normally called in gs_plugin_initialize() and the data should
+ * not be manually freed.
+ *
+ * Returns: the #GsPluginData, cleared to NUL butes
  **/
 GsPluginData *
 gs_plugin_alloc_data (GsPlugin *plugin, gsize sz)
@@ -210,8 +227,10 @@ gs_plugin_alloc_data (GsPlugin *plugin, gsize sz)
 
 /**
  * gs_plugin_action_start:
+ * @plugin: a #GsPlugin
+ * @exclusive: if the plugin action should be performed exclusively
  *
- * FIXME: unexport soon
+ * Starts a plugin action.
  **/
 void
 gs_plugin_action_start (GsPlugin *plugin, gboolean exclusive)
@@ -247,8 +266,9 @@ gs_plugin_action_delay_cb (gpointer user_data)
 
 /**
  * gs_plugin_action_stop:
+ * @plugin: a #GsPlugin
  *
- * FIXME: unexport soon
+ * Stops an plugin action.
  **/
 void
 gs_plugin_action_stop (GsPlugin *plugin)
@@ -277,8 +297,11 @@ gs_plugin_action_stop (GsPlugin *plugin)
 
 /**
  * gs_plugin_get_module:
+ * @plugin: a #GsPlugin
  *
- * FIXME: unexport soon
+ * Gets the external module that backs the plugin.
+ *
+ * Returns: the #GModule, or %NULL
  **/
 GModule *
 gs_plugin_get_module (GsPlugin *plugin)
@@ -289,6 +312,11 @@ gs_plugin_get_module (GsPlugin *plugin)
 
 /**
  * gs_plugin_get_enabled:
+ * @plugin: a #GsPlugin
+ *
+ * Gets if the plugin is enabled.
+ *
+ * Returns: %TRUE if enabled
  **/
 gboolean
 gs_plugin_get_enabled (GsPlugin *plugin)
@@ -299,6 +327,11 @@ gs_plugin_get_enabled (GsPlugin *plugin)
 
 /**
  * gs_plugin_set_enabled:
+ * @plugin: a #GsPlugin
+ * @enabled: the enabled state
+ *
+ * Enables or disables a plugin.
+ * This is normally only called from gs_plugin_initialize().
  **/
 void
 gs_plugin_set_enabled (GsPlugin *plugin, gboolean enabled)
@@ -308,7 +341,12 @@ gs_plugin_set_enabled (GsPlugin *plugin, gboolean enabled)
 }
 
 /**
- * gs_plugin_set_enabled:
+ * gs_plugin_get_name:
+ * @plugin: a #GsPlugin
+ *
+ * Gets the plugin name.
+ *
+ * Returns: a string, e.g. "fwupd"
  **/
 const gchar *
 gs_plugin_get_name (GsPlugin *plugin)
@@ -319,6 +357,11 @@ gs_plugin_get_name (GsPlugin *plugin)
 
 /**
  * gs_plugin_get_scale:
+ * @plugin: a #GsPlugin
+ *
+ * Gets the window scale factor.
+ *
+ * Returns: the factor, usually 1 for standard screens or 2 for HiDPI
  **/
 guint
 gs_plugin_get_scale (GsPlugin *plugin)
@@ -329,6 +372,10 @@ gs_plugin_get_scale (GsPlugin *plugin)
 
 /**
  * gs_plugin_set_scale:
+ * @plugin: a #GsPlugin
+ * @scale: the window scale factor, usually 1 for standard screens or 2 for HiDPI
+ *
+ * Sets the window scale factor.
  **/
 void
 gs_plugin_set_scale (GsPlugin *plugin, guint scale)
@@ -339,6 +386,12 @@ gs_plugin_set_scale (GsPlugin *plugin, guint scale)
 
 /**
  * gs_plugin_get_priority:
+ * @plugin: a #GsPlugin
+ *
+ * Gets the plugin priority, where higher numbers are run after lower
+ * numbers.
+ *
+ * Returns: the integer value
  **/
 guint
 gs_plugin_get_priority (GsPlugin *plugin)
@@ -349,6 +402,11 @@ gs_plugin_get_priority (GsPlugin *plugin)
 
 /**
  * gs_plugin_set_priority:
+ * @plugin: a #GsPlugin
+ * @priority: a integer value
+ *
+ * Sets the plugin priority, where higher numbers are run after lower
+ * numbers.
  **/
 void
 gs_plugin_set_priority (GsPlugin *plugin, guint priority)
@@ -359,6 +417,11 @@ gs_plugin_set_priority (GsPlugin *plugin, guint priority)
 
 /**
  * gs_plugin_get_locale:
+ * @plugin: a #GsPlugin
+ *
+ * Gets the user locale.
+ *
+ * Returns: the locale string, e.g. "en_GB"
  **/
 const gchar *
 gs_plugin_get_locale (GsPlugin *plugin)
@@ -369,6 +432,10 @@ gs_plugin_get_locale (GsPlugin *plugin)
 
 /**
  * gs_plugin_set_locale:
+ * @plugin: a #GsPlugin
+ * @locale: a locale string, e.g. "en_GB"
+ *
+ * Sets the plugin locale.
  **/
 void
 gs_plugin_set_locale (GsPlugin *plugin, const gchar *locale)
@@ -380,6 +447,13 @@ gs_plugin_set_locale (GsPlugin *plugin, const gchar *locale)
 
 /**
  * gs_plugin_get_profile:
+ * @plugin: a #GsPlugin
+ *
+ * Gets the profile object to be used for the plugin.
+ * This can be used to make plugin actions appear in the global profile
+ * output.
+ *
+ * Returns: the #AsProfile
  **/
 AsProfile *
 gs_plugin_get_profile (GsPlugin *plugin)
@@ -390,6 +464,10 @@ gs_plugin_get_profile (GsPlugin *plugin)
 
 /**
  * gs_plugin_set_profile:
+ * @plugin: a #GsPlugin
+ * @profile: a #AsProfile
+ *
+ * Sets the profile object to be used for the plugin.
  **/
 void
 gs_plugin_set_profile (GsPlugin *plugin, AsProfile *profile)
@@ -400,6 +478,11 @@ gs_plugin_set_profile (GsPlugin *plugin, AsProfile *profile)
 
 /**
  * gs_plugin_get_soup_session:
+ * @plugin: a #GsPlugin
+ *
+ * Gets the soup session that plugins can use when downloading.
+ *
+ * Returns: the #SoupSession
  **/
 SoupSession *
 gs_plugin_get_soup_session (GsPlugin *plugin)
@@ -410,6 +493,10 @@ gs_plugin_get_soup_session (GsPlugin *plugin)
 
 /**
  * gs_plugin_set_soup_session:
+ * @plugin: a #GsPlugin
+ * @soup_session: a #SoupSession
+ *
+ * Sets the soup session that plugins will use when downloading.
  **/
 void
 gs_plugin_set_soup_session (GsPlugin *plugin, SoupSession *soup_session)
@@ -420,6 +507,12 @@ gs_plugin_set_soup_session (GsPlugin *plugin, SoupSession *soup_session)
 
 /**
  * gs_plugin_has_flags:
+ * @plugin: a #GsPlugin
+ * @flags: a #GsPluginFlags, e.g. %GS_PLUGIN_FLAGS_RUNNING_SELF
+ *
+ * Finds out if a plugin has a specific flag set.
+ *
+ * Returns: TRUE if the flag is set
  **/
 gboolean
 gs_plugin_has_flags (GsPlugin *plugin, GsPluginFlags flags)
@@ -430,6 +523,10 @@ gs_plugin_has_flags (GsPlugin *plugin, GsPluginFlags flags)
 
 /**
  * gs_plugin_set_running_other:
+ * @plugin: a #GsPlugin
+ * @running_other: %TRUE if another plugin is running
+ *
+ * Inform the plugin that another plugin is running in the loader.
  **/
 void
 gs_plugin_set_running_other (GsPlugin *plugin, gboolean running_other)
@@ -442,48 +539,68 @@ gs_plugin_set_running_other (GsPlugin *plugin, gboolean running_other)
 }
 
 /**
- * gs_plugin_get_order_after:
+ * gs_plugin_add_rule:
+ * @plugin: a #GsPlugin
+ * @rule: a #GsPluginRule, e.g. %GS_PLUGIN_RULE_CONFLICTS
+ * @name: a plugin name, e.g. "appstream"
+ *
+ * If the plugin name is found, the rule will be used to sort the plugin list,
+ * for example the plugin specified by @name will be ordered after this plugin
+ * when %GS_PLUGIN_RULE_RUN_AFTER is used.
+ *
+ * NOTE: The depsolver is iterative and may not solve overly-complicated rules;
+ * If depsolving fails then gnome-software will not start.
  **/
-const gchar **
-gs_plugin_get_order_after (GsPlugin *plugin)
+void
+gs_plugin_add_rule (GsPlugin *plugin, GsPluginRule rule, const gchar *name)
 {
 	GsPluginPrivate *priv = gs_plugin_get_instance_private (plugin);
-	return priv->order_after;
+	g_ptr_array_add (priv->rules[rule], g_strdup (name));
 }
 
 /**
- * gs_plugin_get_order_before:
+ * gs_plugin_get_rules:
+ * @plugin: a #GsPlugin
+ * @rule: a #GsPluginRule, e.g. %GS_PLUGIN_RULE_CONFLICTS
+ *
+ * Gets the plugin IDs that should be run after this plugin.
+ *
+ * Returns: (element-type utf8) (transfer none): the list of plugin names, e.g. ['appstream']
  **/
-const gchar **
-gs_plugin_get_order_before (GsPlugin *plugin)
+GPtrArray *
+gs_plugin_get_rules (GsPlugin *plugin, GsPluginRule rule)
 {
 	GsPluginPrivate *priv = gs_plugin_get_instance_private (plugin);
-	return priv->order_before;
-}
-
-/**
- * gs_plugin_get_conflicts:
- **/
-const gchar **
-gs_plugin_get_conflicts (GsPlugin *plugin)
-{
-	GsPluginPrivate *priv = gs_plugin_get_instance_private (plugin);
-	return priv->conflicts;
+	return priv->rules[rule];
 }
 
 /**
  * gs_plugin_check_distro_id:
+ * @plugin: a #GsPlugin
+ * @distro_id: a distro ID, e.g. "fedora"
+ *
+ * Checks if the distro is compatible.
+ *
+ * Returns: %TRUE if compatible
  **/
 gboolean
 gs_plugin_check_distro_id (GsPlugin *plugin, const gchar *distro_id)
 {
 	g_autoptr(GError) error = NULL;
-	g_autofree gchar *id = NULL;
+	g_autoptr(GsOsRelease) os_release = NULL;
+	const gchar *id = NULL;
+
+	/* load /etc/os-release */
+	os_release = gs_os_release_new (&error);
+	if (os_release == NULL) {
+		g_debug ("could not parse os-release: %s", error->message);
+		return FALSE;
+	}
 
 	/* check that we are running on Fedora */
-	id = gs_os_release_get_id (&error);
+	id = gs_os_release_get_id (os_release);
 	if (id == NULL) {
-		g_debug ("could not parse os-release: %s", error->message);
+		g_debug ("could not get distro ID");
 		return FALSE;
 	}
 	if (g_strcmp0 (id, distro_id) != 0)
@@ -516,6 +633,11 @@ gs_plugin_status_update_cb (gpointer user_data)
 
 /**
  * gs_plugin_status_update:
+ * @plugin: a #GsPlugin
+ * @app: a #GsApp, or %NULL
+ * @status: a #GsPluginStatus, e.g. %GS_PLUGIN_STATUS_DOWNLOADING
+ *
+ * Update the state of the plugin so any UI can be updated.
  **/
 void
 gs_plugin_status_update (GsPlugin *plugin, GsApp *app, GsPluginStatus status)
@@ -552,6 +674,13 @@ gs_plugin_app_launch_cb (gpointer user_data)
 
 /**
  * gs_plugin_app_launch:
+ * @plugin: a #GsPlugin
+ * @app: a #GsApp
+ * @error: a #GError, or %NULL
+ *
+ * Launches the application using GAppInfo.
+ *
+ * Returns: %TRUE for success
  **/
 gboolean
 gs_plugin_app_launch (GsPlugin *plugin, GsApp *app, GError **error)
@@ -568,7 +697,7 @@ gs_plugin_app_launch (GsPlugin *plugin, GsApp *app, GError **error)
 			     desktop_id);
 		return FALSE;
 	}
-	appinfo = G_APP_INFO (g_desktop_app_info_new (desktop_id));
+	appinfo = G_APP_INFO (gs_utils_get_desktop_app_info (desktop_id));
 	if (appinfo == NULL) {
 		g_set_error (error,
 			     GS_PLUGIN_ERROR,
@@ -597,6 +726,10 @@ gs_plugin_updates_changed_cb (gpointer user_data)
 
 /**
  * gs_plugin_updates_changed:
+ * @plugin: a #GsPlugin
+ *
+ * Emit a signal that tells the plugin loader that the list of updates
+ * may have changed.
  **/
 void
 gs_plugin_updates_changed (GsPlugin *plugin)
@@ -658,7 +791,16 @@ gs_plugin_download_chunk_cb (SoupMessage *msg, SoupBuffer *chunk,
 
 /**
  * gs_plugin_download_data:
- */
+ * @plugin: a #GsPlugin
+ * @app: a #GsApp, or %NULL
+ * @uri: a remote URI
+ * @cancellable: a #GCancellable, or %NULL
+ * @error: a #GError, or %NULL
+ *
+ * Downloads data.
+ *
+ * Returns: the downloaded data, or %NULL
+ **/
 GBytes *
 gs_plugin_download_data (GsPlugin *plugin,
 			 GsApp *app,
@@ -696,7 +838,17 @@ gs_plugin_download_data (GsPlugin *plugin,
 
 /**
  * gs_plugin_download_file:
- */
+ * @plugin: a #GsPlugin
+ * @app: a #GsApp, or %NULL
+ * @uri: a remote URI
+ * @filename: a local filename
+ * @cancellable: a #GCancellable, or %NULL
+ * @error: a #GError, or %NULL
+ *
+ * Downloads data and saves it to a file.
+ *
+ * Returns: %TRUE for success
+ **/
 gboolean
 gs_plugin_download_file (GsPlugin *plugin,
 			 GsApp *app,
@@ -746,7 +898,13 @@ gs_plugin_download_file (GsPlugin *plugin,
 
 /**
  * gs_plugin_cache_lookup:
- */
+ * @plugin: a #GsPlugin
+ * @key: a string
+ *
+ * Looks up an application object from the per-plugin cache
+ *
+ * Returns: the #GsApp, or %NULL
+ **/
 GsApp *
 gs_plugin_cache_lookup (GsPlugin *plugin, const gchar *key)
 {
@@ -759,7 +917,13 @@ gs_plugin_cache_lookup (GsPlugin *plugin, const gchar *key)
 
 /**
  * gs_plugin_cache_add:
- */
+ * @plugin: a #GsPlugin
+ * @key: a string
+ * @app: a #GsApp
+ *
+ * Adds an application to the per-plugin cache. This is optional,
+ * and the plugin can use the cache however it likes.
+ **/
 void
 gs_plugin_cache_add (GsPlugin *plugin, const gchar *key, GsApp *app)
 {
@@ -844,6 +1008,11 @@ static void
 gs_plugin_init (GsPlugin *plugin)
 {
 	GsPluginPrivate *priv = gs_plugin_get_instance_private (plugin);
+	guint i;
+
+	for (i = 0; i < GS_PLUGIN_RULE_LAST; i++)
+		priv->rules[i] = g_ptr_array_new_with_free_func (g_free);
+
 	priv->enabled = TRUE;
 	priv->priority = 0.f;
 	priv->scale = 1;
@@ -857,7 +1026,11 @@ gs_plugin_init (GsPlugin *plugin)
 
 /**
  * gs_plugin_new:
- */
+ *
+ * Creates a new plugin.
+ *
+ * Returns: a #GsPlugin
+ **/
 GsPlugin *
 gs_plugin_new (void)
 {
