@@ -50,6 +50,19 @@ struct _GsPluginClass
 
 typedef struct	GsPluginData	GsPluginData;
 
+/**
+ * GsPluginStatus:
+ * @GS_PLUGIN_STATUS_UNKNOWN:		Unknown status
+ * @GS_PLUGIN_STATUS_WAITING:		Waiting
+ * @GS_PLUGIN_STATUS_FINISHED:		Finished
+ * @GS_PLUGIN_STATUS_SETUP:		Setup in progress
+ * @GS_PLUGIN_STATUS_DOWNLOADING:	Downloading in progress
+ * @GS_PLUGIN_STATUS_QUERYING:		Querying in progress
+ * @GS_PLUGIN_STATUS_INSTALLING:	Installing in progress
+ * @GS_PLUGIN_STATUS_REMOVING:		Removing in progress
+ *
+ * The ststus of the plugin.
+ **/
 typedef enum {
 	GS_PLUGIN_STATUS_UNKNOWN,
 	GS_PLUGIN_STATUS_WAITING,
@@ -59,18 +72,41 @@ typedef enum {
 	GS_PLUGIN_STATUS_QUERYING,
 	GS_PLUGIN_STATUS_INSTALLING,
 	GS_PLUGIN_STATUS_REMOVING,
+	/*< private >*/
 	GS_PLUGIN_STATUS_LAST
 } GsPluginStatus;
 
+/**
+ * GsPluginFlags:
+ * @GS_PLUGIN_FLAGS_NONE:		No flags set
+ * @GS_PLUGIN_FLAGS_RUNNING_SELF:	The plugin is running
+ * @GS_PLUGIN_FLAGS_RUNNING_OTHER:	Another plugin is running
+ * @GS_PLUGIN_FLAGS_EXCLUSIVE:		An exclusive action is running
+ * @GS_PLUGIN_FLAGS_RECENT:		This plugin recently ran
+ *
+ * The flags for the plugin at this point in time.
+ **/
 typedef enum {
 	GS_PLUGIN_FLAGS_NONE		= 0,
 	GS_PLUGIN_FLAGS_RUNNING_SELF	= 1 << 0,
 	GS_PLUGIN_FLAGS_RUNNING_OTHER	= 1 << 1,
 	GS_PLUGIN_FLAGS_EXCLUSIVE	= 1 << 2,
 	GS_PLUGIN_FLAGS_RECENT		= 1 << 3,
+	/*< private >*/
 	GS_PLUGIN_FLAGS_LAST
 } GsPluginFlags;
 
+/**
+ * GsPluginError:
+ * @GS_PLUGIN_ERROR_FAILED:		Generic failure
+ * @GS_PLUGIN_ERROR_NOT_SUPPORTED:	Action not supported
+ * @GS_PLUGIN_ERROR_CANCELLED:		Action was cancelled
+ * @GS_PLUGIN_ERROR_NO_NETWORK:		No network connection available
+ * @GS_PLUGIN_ERROR_NO_SECURITY:	Security policy forbid action
+ * @GS_PLUGIN_ERROR_NO_SPACE:		No disk space to allow action
+ *
+ * The failure error types.
+ **/
 typedef enum {
 	GS_PLUGIN_ERROR_FAILED,
 	GS_PLUGIN_ERROR_NOT_SUPPORTED,
@@ -78,9 +114,38 @@ typedef enum {
 	GS_PLUGIN_ERROR_NO_NETWORK,
 	GS_PLUGIN_ERROR_NO_SECURITY,
 	GS_PLUGIN_ERROR_NO_SPACE,
+	/*< private >*/
 	GS_PLUGIN_ERROR_LAST
 } GsPluginError;
 
+/**
+ * GsPluginRefineFlags:
+ * @GS_PLUGIN_REFINE_FLAGS_DEFAULT:			No explicit flags set
+ * @GS_PLUGIN_REFINE_FLAGS_USE_HISTORY:			Get the historical view
+ * @GS_PLUGIN_REFINE_FLAGS_REQUIRE_LICENSE:		Require the license
+ * @GS_PLUGIN_REFINE_FLAGS_REQUIRE_URL:			Require the URL
+ * @GS_PLUGIN_REFINE_FLAGS_REQUIRE_DESCRIPTION:		Require the long description
+ * @GS_PLUGIN_REFINE_FLAGS_REQUIRE_SIZE:		Require the installed and download sizes
+ * @GS_PLUGIN_REFINE_FLAGS_REQUIRE_RATING:		Require the rating
+ * @GS_PLUGIN_REFINE_FLAGS_REQUIRE_VERSION:		Require the version
+ * @GS_PLUGIN_REFINE_FLAGS_REQUIRE_HISTORY:		Require the history
+ * @GS_PLUGIN_REFINE_FLAGS_REQUIRE_SETUP_ACTION:	Require enough to install or remove the package
+ * @GS_PLUGIN_REFINE_FLAGS_REQUIRE_UPDATE_DETAILS:	Require update details
+ * @GS_PLUGIN_REFINE_FLAGS_REQUIRE_ORIGIN:		Require the origin
+ * @GS_PLUGIN_REFINE_FLAGS_REQUIRE_RELATED:		Require related packages
+ * @GS_PLUGIN_REFINE_FLAGS_REQUIRE_MENU_PATH:		Require the menu path
+ * @GS_PLUGIN_REFINE_FLAGS_REQUIRE_ADDONS:		Require available addons
+ * @GS_PLUGIN_REFINE_FLAGS_ALLOW_PACKAGES:		Allow packages to be returned
+ * @GS_PLUGIN_REFINE_FLAGS_REQUIRE_UPDATE_SEVERITY:	Require update severity
+ * @GS_PLUGIN_REFINE_FLAGS_REQUIRE_UPGRADE_REMOVED:	Require distro upgrades
+ * @GS_PLUGIN_REFINE_FLAGS_REQUIRE_PROVENANCE:		Require the provenance
+ * @GS_PLUGIN_REFINE_FLAGS_REQUIRE_REVIEWS:		Require user-reviews
+ * @GS_PLUGIN_REFINE_FLAGS_REQUIRE_REVIEW_RATINGS:	Require user-ratings
+ * @GS_PLUGIN_REFINE_FLAGS_REQUIRE_KEY_COLORS:		Require the key colors
+ * @GS_PLUGIN_REFINE_FLAGS_REQUIRE_ICON:		Require the icon to be loaded
+ *
+ * The refine flags.
+ **/
 typedef enum {
 	GS_PLUGIN_REFINE_FLAGS_DEFAULT			= 0,
 	GS_PLUGIN_REFINE_FLAGS_USE_HISTORY		= 1 << 0,
@@ -103,6 +168,8 @@ typedef enum {
 	GS_PLUGIN_REFINE_FLAGS_REQUIRE_PROVENANCE	= 1 << 17,
 	GS_PLUGIN_REFINE_FLAGS_REQUIRE_REVIEWS		= 1 << 18,
 	GS_PLUGIN_REFINE_FLAGS_REQUIRE_REVIEW_RATINGS	= 1 << 19,
+	GS_PLUGIN_REFINE_FLAGS_REQUIRE_KEY_COLORS	= 1 << 20,
+	GS_PLUGIN_REFINE_FLAGS_REQUIRE_ICON		= 1 << 21,
 	/*< private >*/
 	GS_PLUGIN_REFINE_FLAGS_LAST
 } GsPluginRefineFlags;
@@ -130,6 +197,23 @@ typedef enum {
 	GS_PLUGIN_REFRESH_FLAGS_LAST
 } GsPluginRefreshFlags;
 
+/**
+ * GsPluginRule:
+ * @GS_PLUGIN_RULE_CONFLICTS:		The plugin conflicts with another
+ * @GS_PLUGIN_RULE_RUN_AFTER:		Order the plugin after another
+ * @GS_PLUGIN_RULE_RUN_BEFORE:		Order the plugin before another
+ *
+ * The rules used for ordering plugins.
+ * Plugins are expected to add rules in gs_plugin_initialize().
+ **/
+typedef enum {
+	GS_PLUGIN_RULE_CONFLICTS,
+	GS_PLUGIN_RULE_RUN_AFTER,
+	GS_PLUGIN_RULE_RUN_BEFORE,
+	/*< private >*/
+	GS_PLUGIN_RULE_LAST
+} GsPluginRule;
+
 /* helpers */
 #define	GS_PLUGIN_ERROR					1
 
@@ -147,6 +231,9 @@ guint		 gs_plugin_get_scale			(GsPlugin	*plugin);
 const gchar	*gs_plugin_get_locale			(GsPlugin	*plugin);
 AsProfile	*gs_plugin_get_profile			(GsPlugin	*plugin);
 SoupSession	*gs_plugin_get_soup_session		(GsPlugin	*plugin);
+void		 gs_plugin_add_rule			(GsPlugin	*plugin,
+							 GsPluginRule	 rule,
+							 const gchar	*name);
 
 /* helpers */
 GBytes		*gs_plugin_download_data		(GsPlugin	*plugin,
@@ -176,168 +263,6 @@ gboolean	 gs_plugin_app_launch			(GsPlugin	*plugin,
 void		 gs_plugin_updates_changed		(GsPlugin	*plugin);
 const gchar	*gs_plugin_status_to_string		(GsPluginStatus	 status);
 
-/* vfuncs */
-void		 gs_plugin_initialize			(GsPlugin	*plugin);
-void		 gs_plugin_destroy			(GsPlugin	*plugin);
-void		 gs_plugin_adopt_app			(GsPlugin	*plugin,
-							 GsApp		*app);
-gboolean	 gs_plugin_add_search			(GsPlugin	*plugin,
-							 gchar		**values,
-							 GList		**list,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_add_search_files		(GsPlugin	*plugin,
-							 gchar		**values,
-							 GList		**list,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_add_search_what_provides	(GsPlugin	*plugin,
-							 gchar		**values,
-							 GList		**list,
-							 GCancellable	*cancellable,
-							 GError		**error);
-const gchar	**gs_plugin_order_after			(GsPlugin	*plugin);
-const gchar	**gs_plugin_order_before		(GsPlugin	*plugin);
-const gchar	**gs_plugin_get_conflicts		(GsPlugin	*plugin);
-gboolean	 gs_plugin_setup			(GsPlugin	*plugin,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_add_installed		(GsPlugin	*plugin,
-							 GList		**list,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_add_updates			(GsPlugin	*plugin,
-							 GList		**list,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_add_distro_upgrades		(GsPlugin	*plugin,
-							 GList		**list,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_add_sources			(GsPlugin	*plugin,
-							 GList		**list,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_add_updates_historical	(GsPlugin	*plugin,
-							 GList		**list,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_add_categories		(GsPlugin	*plugin,
-							 GList		**list,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_add_category_apps		(GsPlugin	*plugin,
-							 GsCategory	*category,
-							 GList		**list,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_add_popular			(GsPlugin	*plugin,
-							 GList		**list,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_add_featured			(GsPlugin	*plugin,
-							 GList		**list,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_add_unvoted_reviews		(GsPlugin	*plugin,
-							 GList		**list,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_refine			(GsPlugin	*plugin,
-							 GList		**list,
-							 GsPluginRefineFlags flags,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_refine_app			(GsPlugin	*plugin,
-							 GsApp		*app,
-							 GsPluginRefineFlags flags,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_launch			(GsPlugin	*plugin,
-							 GsApp		*app,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_add_shortcut			(GsPlugin	*plugin,
-							 GsApp		*app,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_remove_shortcut		(GsPlugin	*plugin,
-							 GsApp		*app,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_update_cancel		(GsPlugin	*plugin,
-							 GsApp		*app,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_app_install			(GsPlugin	*plugin,
-							 GsApp		*app,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_app_remove			(GsPlugin	*plugin,
-							 GsApp		*app,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_app_set_rating		(GsPlugin	*plugin,
-							 GsApp		*app,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_update_app			(GsPlugin	*plugin,
-							 GsApp		*app,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_app_upgrade_download		(GsPlugin	*plugin,
-							 GsApp		*app,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_app_upgrade_trigger		(GsPlugin	*plugin,
-							 GsApp		*app,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_review_submit		(GsPlugin	*plugin,
-							 GsApp		*app,
-							 GsReview	*review,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_review_upvote		(GsPlugin	*plugin,
-							 GsApp		*app,
-							 GsReview	*review,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_review_downvote		(GsPlugin	*plugin,
-							 GsApp		*app,
-							 GsReview	*review,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_review_report		(GsPlugin	*plugin,
-							 GsApp		*app,
-							 GsReview	*review,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_review_remove		(GsPlugin	*plugin,
-							 GsApp		*app,
-							 GsReview	*review,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_review_dismiss		(GsPlugin	*plugin,
-							 GsApp		*app,
-							 GsReview	*review,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_refresh			(GsPlugin	*plugin,
-							 guint		 cache_age,
-							 GsPluginRefreshFlags flags,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_file_to_app			(GsPlugin	*plugin,
-							 GList		**list,
-							 GFile		*file,
-							 GCancellable	*cancellable,
-							 GError		**error);
-gboolean	 gs_plugin_update			(GsPlugin	*plugin,
-							 GList		*apps,
-							 GCancellable	*cancellable,
-							 GError		**error);
-
 G_END_DECLS
 
 #endif /* __GS_PLUGIN_H */
