@@ -26,7 +26,7 @@
 
 #include "gs-popular-tile.h"
 #include "gs-star-widget.h"
-#include "gs-utils.h"
+#include "gs-common.h"
 
 struct _GsPopularTile
 {
@@ -103,8 +103,6 @@ app_state_changed (GsApp *app, GParamSpec *pspec, GsPopularTile *tile)
 void
 gs_popular_tile_set_app (GsPopularTile *tile, GsApp *app)
 {
-	const gchar *css;
-
 	g_return_if_fail (GS_IS_POPULAR_TILE (tile));
 	g_return_if_fail (GS_IS_APP (app) || app == NULL);
 
@@ -128,10 +126,9 @@ gs_popular_tile_set_app (GsPopularTile *tile, GsApp *app)
 		 	  G_CALLBACK (app_state_changed), tile);
 	app_state_changed (tile->app, NULL, tile);
 
-	/* set custom css */
-	css = gs_app_get_metadata_item (app, "GnomeSoftware::PopularTile-css");
-	if (css != NULL)
-		gs_utils_widget_set_custom_css (GTK_WIDGET (tile), css);
+	/* perhaps set custom css */
+	gs_utils_widget_set_custom_css (app, GTK_WIDGET (tile),
+					"GnomeSoftware::PopularTile-css");
 
 	gs_image_set_from_pixbuf (GTK_IMAGE (tile->image), gs_app_get_pixbuf (tile->app));
 
