@@ -22,9 +22,13 @@
 #include "config.h"
 
 #include <glib-object.h>
+#include <glib/gstdio.h>
 #include <stdlib.h>
+#include <fnmatch.h>
 
-#include "gs-app.h"
+#include "gs-app-private.h"
+#include "gs-app-list-private.h"
+#include "gs-os-release.h"
 #include "gs-plugin.h"
 #include "gs-plugin-loader.h"
 #include "gs-plugin-loader-sync.h"
@@ -46,6 +50,50 @@ gs_test_get_filename (const gchar *filename)
 	return g_strdup (full_tmp);
 }
 
+/**
+ * gs_test_rmtree:
+ **/
+static gboolean
+gs_test_rmtree (const gchar *directory, GError **error)
+{
+	const gchar *filename;
+	g_autoptr(GDir) dir = NULL;
+
+	/* try to open */
+	dir = g_dir_open (directory, 0, error);
+	if (dir == NULL)
+		return FALSE;
+
+	/* find each */
+	while ((filename = g_dir_read_name (dir))) {
+		g_autofree gchar *src = NULL;
+		src = g_build_filename (directory, filename, NULL);
+		if (g_file_test (src, G_FILE_TEST_IS_DIR) &&
+		    !g_file_test (src, G_FILE_TEST_IS_SYMLINK)) {
+			if (!gs_test_rmtree (src, error))
+				return FALSE;
+		} else {
+			g_debug ("deleting %s", src);
+			if (g_unlink (src) != 0) {
+				g_set_error (error,
+					     GS_PLUGIN_ERROR,
+					     GS_PLUGIN_ERROR_FAILED,
+					     "Failed to delete: %s", src);
+				return FALSE;
+			}
+		}
+	}
+	g_debug ("removing empty %s", directory);
+	if (g_rmdir (directory) != 0) {
+		g_set_error (error,
+			     GS_PLUGIN_ERROR,
+			     GS_PLUGIN_ERROR_FAILED,
+			     "Failed to remove: %s", directory);
+		return FALSE;
+	}
+	return TRUE;
+}
+
 static gboolean
 gs_app_list_filter_cb (GsApp *app, gpointer user_data)
 {
@@ -56,53 +104,76 @@ gs_app_list_filter_cb (GsApp *app, gpointer user_data)
 	return TRUE;
 }
 
+static void
+gs_os_release_func (void)
+{
+	g_autofree gchar *fn = NULL;
+	g_autoptr(GError) error = NULL;
+	g_autoptr(GsOsRelease) os_release = NULL;
+
+	fn = gs_test_get_filename ("tests/os-release");
+	g_assert (fn != NULL);
+	g_setenv ("GS_SELF_TEST_OS_RELEASE_FILENAME", fn, TRUE);
+
+	os_release = gs_os_release_new (&error);
+	g_assert_no_error (error);
+	g_assert (os_release != NULL);
+	g_assert_cmpstr (gs_os_release_get_id (os_release), ==, "fedora");
+	g_assert_cmpstr (gs_os_release_get_name (os_release), ==, "Fedora");
+	g_assert_cmpstr (gs_os_release_get_version (os_release), ==, "25 (Workstation Edition)");
+	g_assert_cmpstr (gs_os_release_get_version_id (os_release), ==, "25");
+	g_assert_cmpstr (gs_os_release_get_pretty_name (os_release), ==, "Fedora 25 (Workstation Edition)");
+}
+
 static void
 gs_plugin_func (void)
 {
-	GList *list = NULL;
-	GList *list_dup;
-	GList *list_remove = NULL;
+	GsAppList *list;
+	GsAppList *list_dup;
+	GsAppList *list_remove;
 	GsApp *app;
 
 	/* add a couple of duplicate IDs */
 	app = gs_app_new ("a");
-	gs_app_list_add (&list, app);
+	list = gs_app_list_new ();
+	gs_app_list_add (list, app);
 	g_object_unref (app);
 
 	/* test refcounting */
-	g_assert_cmpstr (gs_app_get_id (GS_APP (list->data)), ==, "a");
+	g_assert_cmpstr (gs_app_get_id (gs_app_list_index (list, 0)), ==, "a");
 	list_dup = gs_app_list_copy (list);
-	gs_app_list_free (list);
-	g_assert_cmpint (g_list_length (list_dup), ==, 1);
-	g_assert_cmpstr (gs_app_get_id (GS_APP (list_dup->data)), ==, "a");
-	gs_app_list_free (list_dup);
+	g_object_unref (list);
+	g_assert_cmpint (gs_app_list_length (list_dup), ==, 1);
+	g_assert_cmpstr (gs_app_get_id (gs_app_list_index (list_dup, 0)), ==, "a");
+	g_object_unref (list_dup);
 
 	/* test removing obects */
 	app = gs_app_new ("a");
-	gs_app_list_add (&list_remove, app);
+	list_remove = gs_app_list_new ();
+	gs_app_list_add (list_remove, app);
 	g_object_unref (app);
 	app = gs_app_new ("b");
-	gs_app_list_add (&list_remove, app);
+	gs_app_list_add (list_remove, app);
 	g_object_unref (app);
 	app = gs_app_new ("c");
-	gs_app_list_add (&list_remove, app);
+	gs_app_list_add (list_remove, app);
 	g_object_unref (app);
-	g_assert_cmpint (g_list_length (list_remove), ==, 3);
-	gs_app_list_filter (&list_remove, gs_app_list_filter_cb, NULL);
-	g_assert_cmpint (g_list_length (list_remove), ==, 1);
-	g_assert_cmpstr (gs_app_get_id (GS_APP (list_remove->data)), ==, "b");
+	g_assert_cmpint (gs_app_list_length (list_remove), ==, 3);
+	gs_app_list_filter (list_remove, gs_app_list_filter_cb, NULL);
+	g_assert_cmpint (gs_app_list_length (list_remove), ==, 1);
+	g_assert_cmpstr (gs_app_get_id (gs_app_list_index (list_remove, 0)), ==, "b");
 
 	/* test removing duplicates */
 	app = gs_app_new ("b");
-	gs_app_list_add (&list_remove, app);
+	gs_app_list_add (list_remove, app);
 	g_object_unref (app);
 	app = gs_app_new ("b");
-	gs_app_list_add (&list_remove, app);
+	gs_app_list_add (list_remove, app);
 	g_object_unref (app);
-	gs_app_list_filter_duplicates (&list_remove);
-	g_assert_cmpint (g_list_length (list_remove), ==, 1);
-	g_assert_cmpstr (gs_app_get_id (GS_APP (list_remove->data)), ==, "b");
-	gs_app_list_free (list_remove);
+	gs_app_list_filter_duplicates (list_remove);
+	g_assert_cmpint (gs_app_list_length (list_remove), ==, 1);
+	g_assert_cmpstr (gs_app_get_id (gs_app_list_index (list_remove, 0)), ==, "b");
+	g_object_unref (list_remove);
 }
 
 static void
@@ -110,10 +181,11 @@ gs_app_func (void)
 {
 	g_autoptr(GsApp) app = NULL;
 
-	app = gs_app_new ("gnome-software");
+	app = gs_app_new ("flatpak:gnome-software");
 	g_assert (GS_IS_APP (app));
 
-	g_assert_cmpstr (gs_app_get_id (app), ==, "gnome-software");
+	g_assert_cmpstr (gs_app_get_id (app), ==, "flatpak:gnome-software");
+	g_assert_cmpstr (gs_app_get_id_no_prefix (app), ==, "gnome-software");
 
 	/* check we clean up the version, but not at the expense of having
 	 * the same string as the update version */
@@ -238,12 +310,29 @@ gs_plugin_loader_refine_func (GsPluginLoader *plugin_loader)
 	g_assert_no_error (error);
 	g_assert (ret);
 
-	g_assert_cmpstr (gs_app_get_license (app), ==,
-			 "<a href=\"http://spdx.org/licenses/GPL-2.0+\">GPL-2.0+</a>");
+	g_assert_cmpstr (gs_app_get_license (app), ==, "GPL-2.0+");
 	g_assert_cmpstr (gs_app_get_description (app), !=, NULL);
 	g_assert_cmpstr (gs_app_get_url (app, AS_URL_KIND_HOMEPAGE), ==, "http://www.test.org/");
 }
 
+static void
+gs_plugin_loader_key_colors_func (GsPluginLoader *plugin_loader)
+{
+	gboolean ret;
+	g_autoptr(GsApp) app = NULL;
+	g_autoptr(GError) error = NULL;
+
+	/* get the extra bits */
+	app = gs_app_new ("zeus.desktop");
+	ret = gs_plugin_loader_app_refine (plugin_loader, app,
+					   GS_PLUGIN_REFINE_FLAGS_REQUIRE_KEY_COLORS,
+					   NULL,
+					   &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+	g_assert_cmpint (gs_app_get_key_colors(app)->len, >=, 3);
+}
+
 static void
 gs_plugin_loader_updates_func (GsPluginLoader *plugin_loader)
 {
@@ -260,8 +349,8 @@ gs_plugin_loader_updates_func (GsPluginLoader *plugin_loader)
 	g_assert (list != NULL);
 
 	/* make sure there are two entries */
-	g_assert_cmpint (g_list_length (list), ==, 2);
-	app = g_list_nth_data (list, 0);
+	g_assert_cmpint (gs_app_list_length (list), ==, 2);
+	app = gs_app_list_index (list, 0);
 	g_assert_cmpstr (gs_app_get_id (app), ==, "chiron.desktop");
 	g_assert_cmpint (gs_app_get_kind (app), ==, AS_APP_KIND_DESKTOP);
 	g_assert_cmpint (gs_app_get_state (app), ==, AS_APP_STATE_UPDATABLE_LIVE);
@@ -269,7 +358,7 @@ gs_plugin_loader_updates_func (GsPluginLoader *plugin_loader)
 	g_assert_cmpint (gs_app_get_update_urgency (app), ==, AS_URGENCY_KIND_HIGH);
 
 	/* get the virtual non-apps OS update */
-	app = g_list_nth_data (list, 1);
+	app = gs_app_list_index (list, 1);
 	g_assert_cmpstr (gs_app_get_id (app), ==, "os-update.virtual");
 	g_assert_cmpstr (gs_app_get_name (app), ==, "OS Updates");
 	g_assert_cmpstr (gs_app_get_summary (app), ==, "Includes performance, stability and security improvements.");
@@ -295,8 +384,8 @@ gs_plugin_loader_distro_upgrades_func (GsPluginLoader *plugin_loader)
 	g_assert (list != NULL);
 
 	/* make sure there is one entry */
-	g_assert_cmpint (g_list_length (list), ==, 1);
-	app = GS_APP (list->data);
+	g_assert_cmpint (gs_app_list_length (list), ==, 1);
+	app = gs_app_list_index (list, 0);
 	g_assert_cmpstr (gs_app_get_id (app), ==, "org.fedoraproject.release-rawhide.upgrade");
 	g_assert_cmpint (gs_app_get_kind (app), ==, AS_APP_KIND_OS_UPGRADE);
 	g_assert_cmpint (gs_app_get_state (app), ==, AS_APP_STATE_AVAILABLE);
@@ -338,6 +427,8 @@ gs_plugin_loader_installed_func (GsPluginLoader *plugin_loader)
 
 	/* get installed packages */
 	list = gs_plugin_loader_get_installed (plugin_loader,
+					       GS_PLUGIN_REFINE_FLAGS_REQUIRE_ORIGIN |
+					       GS_PLUGIN_REFINE_FLAGS_REQUIRE_ADDONS |
 					       GS_PLUGIN_REFINE_FLAGS_REQUIRE_LICENSE |
 					       GS_PLUGIN_REFINE_FLAGS_REQUIRE_MENU_PATH |
 					       GS_PLUGIN_REFINE_FLAGS_REQUIRE_PROVENANCE,
@@ -347,8 +438,8 @@ gs_plugin_loader_installed_func (GsPluginLoader *plugin_loader)
 	g_assert (list != NULL);
 
 	/* make sure there is one entry */
-	g_assert_cmpint (g_list_length (list), ==, 1);
-	app = GS_APP (list->data);
+	g_assert_cmpint (gs_app_list_length (list), ==, 1);
+	app = gs_app_list_index (list, 0);
 	g_assert_cmpstr (gs_app_get_id (app), ==, "zeus.desktop");
 	g_assert_cmpint (gs_app_get_kind (app), ==, AS_APP_KIND_DESKTOP);
 	g_assert_cmpint (gs_app_get_state (app), ==, AS_APP_STATE_INSTALLED);
@@ -358,8 +449,7 @@ gs_plugin_loader_installed_func (GsPluginLoader *plugin_loader)
 
 	/* check various bitfields */
 	g_assert (gs_app_has_quirk (app, AS_APP_QUIRK_PROVENANCE));
-	g_assert_cmpstr (gs_app_get_license (app), ==,
-			 "<a href=\"http://spdx.org/licenses/GPL-2.0+\">GPL-2.0+</a>");
+	g_assert_cmpstr (gs_app_get_license (app), ==, "GPL-2.0+");
 	g_assert (gs_app_get_license_is_free (app));
 
 	/* check kudos */
@@ -384,6 +474,8 @@ gs_plugin_loader_installed_func (GsPluginLoader *plugin_loader)
 	g_assert_cmpint (gs_app_get_state (addon), ==, AS_APP_STATE_UNKNOWN);
 	g_assert_cmpstr (gs_app_get_name (addon), ==, "Spell Check");
 	g_assert_cmpstr (gs_app_get_source_default (addon), ==, "zeus-spell");
+	g_assert_cmpstr (gs_app_get_license (addon), ==,
+			 "LicenseRef-free=https://www.debian.org/");
 	g_assert (gs_app_get_pixbuf (addon) == NULL);
 }
 
@@ -405,8 +497,8 @@ gs_plugin_loader_search_func (GsPluginLoader *plugin_loader)
 	g_assert (list != NULL);
 
 	/* make sure there is one entry, the parent app */
-	g_assert_cmpint (g_list_length (list), ==, 1);
-	app = GS_APP (list->data);
+	g_assert_cmpint (gs_app_list_length (list), ==, 1);
+	app = gs_app_list_index (list, 0);
 	g_assert_cmpstr (gs_app_get_id (app), ==, "zeus.desktop");
 	g_assert_cmpint (gs_app_get_kind (app), ==, AS_APP_KIND_DESKTOP);
 }
@@ -422,7 +514,7 @@ gs_plugin_loader_webapps_func (GsPluginLoader *plugin_loader)
 	app = gs_app_new ("arachne.desktop");
 	gs_app_set_kind (app, AS_APP_KIND_WEB_APP);
 	ret = gs_plugin_loader_app_refine (plugin_loader, app,
-					   GS_PLUGIN_REFINE_FLAGS_DEFAULT,
+					   GS_PLUGIN_REFINE_FLAGS_REQUIRE_ICON,
 					   NULL,
 					   &error);
 	g_assert_no_error (error);
@@ -466,9 +558,249 @@ gs_plugin_loader_dpkg_func (GsPluginLoader *plugin_loader)
 	g_assert (gs_app_get_local_file (app) != NULL);
 }
 
+static void
+gs_plugin_loader_packagekit_local_func (GsPluginLoader *plugin_loader)
+{
+	g_autoptr(GsApp) app = NULL;
+	g_autoptr(GError) error = NULL;
+	g_autofree gchar *fn = NULL;
+	g_autoptr(GFile) file = NULL;
+
+	/* no dpkg, abort */
+	if (!gs_plugin_loader_get_enabled (plugin_loader, "packagekit-local"))
+		return;
+
+	/* load local file */
+	fn = gs_test_get_filename ("tests/chiron-1.1-1.fc24.x86_64.rpm");
+	g_assert (fn != NULL);
+	file = g_file_new_for_path (fn);
+	app = gs_plugin_loader_file_to_app (plugin_loader,
+					    file,
+					    GS_PLUGIN_REFINE_FLAGS_DEFAULT,
+					    NULL,
+					    &error);
+	g_assert_no_error (error);
+	g_assert (app != NULL);
+	g_assert_cmpstr (gs_app_get_id (app), ==, NULL);
+	g_assert_cmpstr (gs_app_get_source_default (app), ==, "chiron");
+	g_assert_cmpstr (gs_app_get_url (app, AS_URL_KIND_HOMEPAGE), ==, "http://127.0.0.1/");
+	g_assert_cmpstr (gs_app_get_name (app), ==, "chiron");
+	g_assert_cmpstr (gs_app_get_version (app), ==, "1.1-1.fc24");
+	g_assert_cmpstr (gs_app_get_summary (app), ==, "Single line synopsis");
+	g_assert_cmpstr (gs_app_get_description (app), ==,
+			 "This is the first paragraph in the example "
+			 "package spec file.  This is the second paragraph.");
+}
+
+static void
+gs_plugin_loader_fwupd_func (GsPluginLoader *plugin_loader)
+{
+	g_autoptr(GsApp) app = NULL;
+	g_autoptr(GError) error = NULL;
+	g_autofree gchar *fn = NULL;
+	g_autoptr(GFile) file = NULL;
+
+	/* no dpkg, abort */
+	if (!gs_plugin_loader_get_enabled (plugin_loader, "fwupd"))
+		return;
+
+	/* load local file */
+	fn = gs_test_get_filename ("tests/chiron-0.2.cab");
+	g_assert (fn != NULL);
+	file = g_file_new_for_path (fn);
+	app = gs_plugin_loader_file_to_app (plugin_loader,
+					    file,
+					    GS_PLUGIN_REFINE_FLAGS_DEFAULT,
+					    NULL,
+					    &error);
+	g_assert_no_error (error);
+	g_assert (app != NULL);
+	g_assert_cmpint (gs_app_get_kind (app), ==, AS_APP_KIND_FIRMWARE);
+	g_assert (gs_app_get_pixbuf (app) != NULL);
+	g_assert (gs_app_get_license (app) != NULL);
+	g_assert (gs_app_has_category (app, "System"));
+	g_assert_cmpstr (gs_app_get_url (app, AS_URL_KIND_HOMEPAGE), ==, "http://127.0.0.1/");
+	g_assert_cmpstr (gs_app_get_name (app), ==, "Chiron");
+	g_assert_cmpstr (gs_app_get_summary (app), ==, "Single line synopsis");
+	g_assert_cmpstr (gs_app_get_version (app), ==, "0.2");
+	g_assert_cmpint (gs_app_get_size_download (app), ==, 32784);
+	g_assert_cmpstr (gs_app_get_description (app), ==,
+			 "This is the first paragraph in the example "
+			 "cab file.\n\nThis is the second paragraph.");
+	g_assert_cmpstr (gs_app_get_update_details (app), ==,
+			 "Latest firmware release.");
+
+	/* seems wrong, but this is only set if the update is available */
+	g_assert_cmpint (gs_app_get_state (app), ==, AS_APP_STATE_UNKNOWN);
+	g_assert_cmpstr (gs_app_get_id (app), ==, NULL);
+}
+
+static void
+gs_plugin_loader_flatpak_func (GsPluginLoader *plugin_loader)
+{
+	GsApp *app;
+	const gchar *root;
+	gboolean ret;
+	gint kf_remote_repo_version;
+	g_autofree gchar *changed_fn = NULL;
+	g_autofree gchar *config_fn = NULL;
+	g_autofree gchar *desktop_fn = NULL;
+	g_autofree gchar *kf_remote_url = NULL;
+	g_autofree gchar *metadata_fn = NULL;
+	g_autofree gchar *runtime_fn = NULL;
+	g_autofree gchar *testdir = NULL;
+	g_autofree gchar *testdir_repourl = NULL;
+	g_autoptr(GError) error = NULL;
+	g_autoptr(GKeyFile) kf1 = g_key_file_new ();
+	g_autoptr(GKeyFile) kf2 = g_key_file_new ();
+	g_autoptr(GsApp) app_source = NULL;
+	g_autoptr(GsAppList) list = NULL;
+	g_autoptr(GsAppList) sources = NULL;
+
+	/* no flatpak, abort */
+	if (!gs_plugin_loader_get_enabled (plugin_loader, "flatpak"))
+		return;
+
+	/* check changed file exists */
+	root = g_getenv ("GS_SELF_TEST_FLATPACK_DATADIR");
+	changed_fn = g_build_filename (root, "flatpak", ".changed", NULL);
+	g_assert (g_file_test (changed_fn, G_FILE_TEST_IS_REGULAR));
+
+	/* check repo is set up */
+	config_fn = g_build_filename (root, "flatpak", "repo", "config", NULL);
+	ret = g_key_file_load_from_file (kf1, config_fn, G_KEY_FILE_NONE, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+	kf_remote_repo_version = g_key_file_get_integer (kf1, "core", "repo_version", &error);
+	g_assert_no_error (error);
+	g_assert_cmpint (kf_remote_repo_version, ==, 1);
+
+	/* add a remote */
+	app_source = gs_app_new ("test");
+	testdir = gs_test_get_filename ("tests/flatpak");
+	if (testdir == NULL)
+		return;
+	testdir_repourl = g_strdup_printf ("file://%s/repo", testdir);
+	gs_app_set_kind (app_source, AS_APP_KIND_SOURCE);
+	gs_app_set_management_plugin (app_source, "flatpak");
+	gs_app_set_state (app_source, AS_APP_STATE_AVAILABLE);
+	gs_app_set_url (app_source, AS_URL_KIND_HOMEPAGE, testdir_repourl);
+	ret = gs_plugin_loader_app_action (plugin_loader, app_source,
+					   GS_PLUGIN_LOADER_ACTION_ADD_SOURCE,
+					   NULL,
+					   &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+	g_assert_cmpint (gs_app_get_state (app_source), ==, AS_APP_STATE_INSTALLED);
+
+	/* check remote was set up */
+	ret = g_key_file_load_from_file (kf2, config_fn, G_KEY_FILE_NONE, &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+	kf_remote_url = g_key_file_get_string (kf2, "remote \"test\"", "url", &error);
+	g_assert_no_error (error);
+	g_assert_cmpstr (kf_remote_url, !=, NULL);
+
+	/* check the source now exists */
+	sources = gs_plugin_loader_get_sources (plugin_loader,
+						GS_PLUGIN_REFINE_FLAGS_DEFAULT,
+						NULL,
+						&error);
+	g_assert_no_error (error);
+	g_assert (sources != NULL);
+	g_assert_cmpint (gs_app_list_length (sources), ==, 1);
+	app = gs_app_list_index (sources, 0);
+	g_assert_cmpstr (gs_app_get_id (app), ==, "test");
+	g_assert_cmpint (gs_app_get_kind (app), ==, AS_APP_KIND_SOURCE);
+
+	/* refresh the appstream metadata */
+	ret = gs_plugin_loader_refresh (plugin_loader,
+					G_MAXUINT,
+					GS_PLUGIN_REFRESH_FLAGS_METADATA,
+					NULL,
+					&error);
+	g_assert_no_error (error);
+	g_assert (ret);
+
+	/* find available application */
+	list = gs_plugin_loader_search (plugin_loader,
+					"Bingo",
+					GS_PLUGIN_REFINE_FLAGS_DEFAULT,
+					NULL,
+					&error);
+	g_assert_no_error (error);
+	g_assert (list != NULL);
+
+	/* make sure there is one entry, the flatpak app */
+	g_assert_cmpint (gs_app_list_length (list), ==, 1);
+	app = gs_app_list_index (list, 0);
+	g_assert_cmpstr (gs_app_get_id (app), ==, "org.test.Chiron.desktop");
+	g_assert_cmpint (gs_app_get_kind (app), ==, AS_APP_KIND_DESKTOP);
+	g_assert_cmpint (gs_app_get_state (app), ==, AS_APP_STATE_AVAILABLE);
+
+	/* install, also installing runtime */
+	ret = gs_plugin_loader_app_action (plugin_loader, app,
+					   GS_PLUGIN_LOADER_ACTION_INSTALL,
+					   NULL,
+					   &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+	g_assert_cmpint (gs_app_get_state (app), ==, AS_APP_STATE_INSTALLED);
+
+	/* check the application exists in the right places */
+	metadata_fn = g_build_filename (root,
+					"flatpak",
+					"app",
+					"org.test.Chiron",
+					"current",
+					"active",
+					"metadata",
+					NULL);
+	g_assert (g_file_test (metadata_fn, G_FILE_TEST_IS_REGULAR));
+	desktop_fn = g_build_filename (root,
+					"flatpak",
+					"app",
+					"org.test.Chiron",
+					"current",
+					"active",
+					"export",
+					"share",
+					"applications",
+					"org.test.Chiron.desktop",
+					NULL);
+	g_assert (g_file_test (desktop_fn, G_FILE_TEST_IS_REGULAR));
+
+	/* check the runtime was installed as well */
+	runtime_fn = g_build_filename (root,
+					"flatpak",
+					"runtime",
+					"org.test.Runtime",
+					"x86_64",
+					"master",
+					"active",
+					"files",
+					"share",
+					"libtest",
+					"README",
+					NULL);
+	g_assert (g_file_test (runtime_fn, G_FILE_TEST_IS_REGULAR));
+
+	/* remove the application */
+	ret = gs_plugin_loader_app_action (plugin_loader, app,
+					   GS_PLUGIN_LOADER_ACTION_REMOVE,
+					   NULL,
+					   &error);
+	g_assert_no_error (error);
+	g_assert (ret);
+	g_assert_cmpint (gs_app_get_state (app), ==, AS_APP_STATE_AVAILABLE);
+	g_assert (!g_file_test (metadata_fn, G_FILE_TEST_IS_REGULAR));
+	g_assert (!g_file_test (desktop_fn, G_FILE_TEST_IS_REGULAR));
+}
+
 int
 main (int argc, char **argv)
 {
+	const gchar *tmp_root = "/var/tmp/self-test";
 	gboolean ret;
 	g_autofree gchar *fn = NULL;
 	g_autofree gchar *xml = NULL;
@@ -479,10 +811,15 @@ main (int argc, char **argv)
 		"dpkg",
 		"dummy",
 		"epiphany",
+		"flatpak",
+		"fwupd",
 		"hardcoded-blacklist",
 		"icons",
 		"menu-spec-refine",
+		"key-colors",
 		"provenance",
+		"provenance-license",
+		"packagekit-local",
 		NULL
 	};
 
@@ -493,6 +830,17 @@ main (int argc, char **argv)
 	g_setenv ("GS_SELF_TEST_LOCALE", "en_GB", TRUE);
 	g_setenv ("GS_SELF_TEST_DUMMY_ENABLE", "1", TRUE);
 	g_setenv ("GS_SELF_TEST_PROVENANCE_SOURCES", "london*,boston", TRUE);
+	g_setenv ("GS_SELF_TEST_PROVENANCE_LICENSE_SOURCES", "london*,boston", TRUE);
+	g_setenv ("GS_SELF_TEST_PROVENANCE_LICENSE_URL", "https://www.debian.org/", TRUE);
+	g_setenv ("GS_SELF_TEST_FLATPACK_DATADIR", tmp_root, TRUE);
+
+	/* ensure test root does not exist */
+	if (g_file_test (tmp_root, G_FILE_TEST_EXISTS)) {
+		ret = gs_test_rmtree (tmp_root, &error);
+		g_assert_no_error (error);
+		g_assert (ret);
+		g_assert (!g_file_test (tmp_root, G_FILE_TEST_EXISTS));
+	}
 
 	fn = gs_test_get_filename ("icons/hicolor/48x48/org.gnome.Software.png");
 	g_assert (fn != NULL);
@@ -542,13 +890,30 @@ main (int argc, char **argv)
 		"    <name>test</name>\n"
 		"    <icon type=\"remote\">file://%s</icon>\n"
 		"  </component>\n"
+		"  <component type=\"desktop\">\n"
+		"    <id>org.test.Chiron.desktop</id>\n"
+		"    <name>Chiron</name>\n"
+		"    <summary>Single line synopsis</summary>\n"
+		"    <description><p>Long description.</p></description>\n"
+		"    <icon height=\"128\" width=\"128\" type=\"cached\">128x128/org.test.Chiron.png</icon>\n"
+		"    <icon height=\"64\" width=\"64\" type=\"cached\">64x64/org.test.Chiron.png</icon>\n"
+		"    <keywords>\n"
+		"      <keyword>Bingo</keyword>\n"
+		"    </keywords>\n"
+		"    <project_license>GPL-2.0+</project_license>\n"
+		"    <url type=\"homepage\">http://127.0.0.1/</url>\n"
+		"    <bundle type=\"flatpak\" runtime=\"org.test.Runtime/x86_64/master\">app/org.test.Chiron/x86_64/master</bundle>\n"
+		"  </component>\n"
 		"</components>\n", fn);
 	g_setenv ("GS_SELF_TEST_APPSTREAM_XML", xml, TRUE);
+	g_setenv ("GS_SELF_TEST_APPSTREAM_ICON_ROOT",
+		  "/var/tmp/self-test/flatpak/appstream/test/x86_64/active/", TRUE);
 
 	/* only critical and error are fatal */
 	g_log_set_fatal_mask (NULL, G_LOG_LEVEL_ERROR | G_LOG_LEVEL_CRITICAL);
 
 	/* generic tests go here */
+	g_test_add_func ("/gnome-software/os-release", gs_os_release_func);
 	g_test_add_func ("/gnome-software/app", gs_app_func);
 	g_test_add_func ("/gnome-software/plugin", gs_plugin_func);
 
@@ -567,6 +932,18 @@ main (int argc, char **argv)
 	g_assert (gs_plugin_loader_get_enabled (plugin_loader, "dummy"));
 
 	/* plugin tests go here */
+	g_test_add_data_func ("/gnome-software/plugin-loader{flatpak}",
+			      plugin_loader,
+			      (GTestDataFunc) gs_plugin_loader_flatpak_func);
+	g_test_add_data_func ("/gnome-software/plugin-loader{fwupd}",
+			      plugin_loader,
+			      (GTestDataFunc) gs_plugin_loader_fwupd_func);
+	g_test_add_data_func ("/gnome-software/plugin-loader{key-colors}",
+			      plugin_loader,
+			      (GTestDataFunc) gs_plugin_loader_key_colors_func);
+	g_test_add_data_func ("/gnome-software/plugin-loader{packagekit-local}",
+			      plugin_loader,
+			      (GTestDataFunc) gs_plugin_loader_packagekit_local_func);
 	g_test_add_data_func ("/gnome-software/plugin-loader{dpkg}",
 			      plugin_loader,
 			      (GTestDataFunc) gs_plugin_loader_dpkg_func);
