@@ -24,11 +24,11 @@
 
 #include <string.h>
 #include <glib/gi18n.h>
-#include <appstream-glib.h>
 
-#include "gs-utils.h"
+#include "gs-common.h"
 
 #include "gs-shell-details.h"
+#include "gs-app-private.h"
 #include "gs-app-addon-row.h"
 #include "gs-history-dialog.h"
 #include "gs-screenshot-image.h"
@@ -312,8 +312,7 @@ gs_shell_details_switch_to (GsPage *page, gboolean scroll_up)
 	case AS_APP_STATE_INSTALLED:
 	case AS_APP_STATE_UPDATABLE:
 	case AS_APP_STATE_UPDATABLE_LIVE:
-		if (gs_app_get_kind (self->app) == AS_APP_KIND_DESKTOP ||
-		    gs_app_get_kind (self->app) == AS_APP_KIND_WEB_APP) {
+		if (!gs_app_has_quirk (self->app, AS_APP_QUIRK_NOT_LAUNCHABLE)) {
 			gtk_widget_set_visible (self->button_details_launch, TRUE);
 		} else {
 			gtk_widget_set_visible (self->button_details_launch, FALSE);
@@ -668,6 +667,105 @@ gs_shell_details_history_cb (GtkLabel *label,
 	return TRUE;
 }
 
+/**
+ * gs_shell_details_get_license_markup:
+ **/
+static gchar *
+gs_shell_details_get_license_markup (const gchar *license)
+{
+	GString *urld;
+	guint i;
+	g_auto(GStrv) tokens = NULL;
+
+	/* URLify any SPDX IDs */
+	urld = g_string_new ("");
+	tokens = as_utils_spdx_license_tokenize (license);
+	for (i = 0; tokens[i] != NULL; i++) {
+
+		/* translated join */
+		if (g_strcmp0 (tokens[i], "&") == 0) {
+			/* TRANSLATORS: This is how we join the licenses and can
+			 * be considered a "Conjunctive AND Operator" according
+			 * to the SPDX specification. For example:
+			 * "LGPL-2.1 and MIT and BSD-2-Clause" */
+			g_string_append (urld, _(" and "));
+			continue;
+		}
+		if (g_strcmp0 (tokens[i], "|") == 0) {
+			/* TRANSLATORS: This is how we join the licenses and can
+			 * be considered a "Disjunctive OR Operator" according
+			 * to the SPDX specification. For example:
+			 * "LGPL-2.1 or MIT" */
+			g_string_append (urld, _(" or "));
+			continue;
+		}
+
+		/* legacy literal text */
+		if (g_str_has_prefix (tokens[i], "#")) {
+			g_string_append (urld, tokens[i] + 1);
+			continue;
+		}
+
+		/* proprietary software */
+		if (g_strcmp0 (tokens[i], "@LicenseRef-proprietary") == 0) {
+			const gchar *url = "https://en.wikipedia.org/wiki/Proprietary_software";
+			g_string_append_printf (urld,
+						"<a href=\"%s\">%s</a>",
+						/* TRANSLATORS: non-free app */
+						url, _("Proprietary"));
+			continue;
+		}
+
+		/* public domain */
+		if (g_strcmp0 (tokens[i], "@LicenseRef-public-domain") == 0) {
+			const gchar *url = "https://en.wikipedia.org/wiki/Public_domain";
+			g_string_append_printf (urld,
+						"<a href=\"%s\">%s</a>",
+						/* TRANSLATORS: see the wikipedia page */
+						url, _("Public domain"));
+			continue;
+		}
+
+		/* free software, license unspecified */
+		if (g_str_has_prefix (tokens[i], "@LicenseRef-free")) {
+			const gchar *url = "https://www.gnu.org/philosophy/free-sw";
+			gchar *tmp;
+
+			/* we support putting a custom URL in the
+			 * token string, e.g. @LicenseRef-free=http://ubuntu.com */
+			tmp = g_strstr_len (tokens[i], -1, "=");
+			if (tmp != NULL)
+				url = tmp + 1;
+			g_string_append_printf (urld,
+						"<a href=\"%s\">%s</a>",
+						/* TRANSLATORS: see GNU page */
+						url, _("Free Software"));
+			continue;
+		}
+
+		/* SPDX value */
+		if (g_str_has_prefix (tokens[i], "@")) {
+			g_string_append_printf (urld,
+						"<a href=\"http://spdx.org/licenses/%s\">%s</a>",
+						tokens[i] + 1, tokens[i] + 1);
+			continue;
+		}
+
+		/* new SPDX value the extractor didn't know about */
+		if (as_utils_is_spdx_license_id (tokens[i])) {
+			g_string_append_printf (urld,
+						"<a href=\"http://spdx.org/licenses/%s\">%s</a>",
+						tokens[i], tokens[i]);
+			continue;
+		}
+
+		/* unknown value */
+		g_string_append (urld, tokens[i]);
+	}
+
+	return g_string_free (urld, FALSE);
+}
+
 /**
  * gs_shell_details_refresh_all:
  **/
@@ -743,7 +841,10 @@ gs_shell_details_refresh_all (GsShellDetails *self)
 		gtk_label_set_label (GTK_LABEL (self->label_details_license_value), C_("license", "Unknown"));
 		gtk_widget_set_tooltip_text (self->label_details_license_value, NULL);
 	} else {
-		gtk_label_set_markup (GTK_LABEL (self->label_details_license_value), tmp);
+		g_autofree gchar *license_markup = NULL;
+		license_markup = gs_shell_details_get_license_markup (tmp);
+		gtk_label_set_markup (GTK_LABEL (self->label_details_license_value),
+				      license_markup);
 		gtk_widget_set_tooltip_text (self->label_details_license_value, NULL);
 	}
 
@@ -1273,8 +1374,8 @@ gs_shell_details_file_to_app_cb (GObject *source,
 		const gchar *msg;
 
 		if (g_error_matches (error,
-				     GS_PLUGIN_LOADER_ERROR,
-				     GS_PLUGIN_LOADER_ERROR_NO_RESULTS)) {
+				     GS_PLUGIN_ERROR,
+				     GS_PLUGIN_ERROR_NOT_SUPPORTED)) {
 			/* TRANSLATORS: the file format was not recognised by
 			 * any plugin, e.g. if you try installing a .tar.gz */
 			msg = _("The file is not supported.");
@@ -1336,7 +1437,7 @@ gs_shell_details_set_filename (GsShellDetails *self, const gchar *filename)
 	file = g_file_new_for_path (filename);
 	gs_plugin_loader_file_to_app_async (self->plugin_loader,
 					    file,
-					    GS_PLUGIN_REFINE_FLAGS_DEFAULT |
+					    GS_PLUGIN_REFINE_FLAGS_REQUIRE_ICON |
 					    GS_PLUGIN_REFINE_FLAGS_REQUIRE_RATING |
 					    GS_PLUGIN_REFINE_FLAGS_REQUIRE_REVIEW_RATINGS |
 					    GS_PLUGIN_REFINE_FLAGS_REQUIRE_REVIEWS,
@@ -1352,6 +1453,7 @@ static void
 gs_shell_details_load (GsShellDetails *self)
 {
 	gs_plugin_loader_app_refine_async (self->plugin_loader, self->app,
+					   GS_PLUGIN_REFINE_FLAGS_REQUIRE_ICON |
 					   GS_PLUGIN_REFINE_FLAGS_REQUIRE_LICENSE |
 					   GS_PLUGIN_REFINE_FLAGS_REQUIRE_SIZE |
 					   GS_PLUGIN_REFINE_FLAGS_REQUIRE_RATING |
