@@ -27,7 +27,7 @@
 #include "gs-app-row.h"
 #include "gs-language.h"
 #include "gs-shell.h"
-#include "gs-utils.h"
+#include "gs-common.h"
 #include "gs-vendor.h"
 
 #include <glib/gi18n.h>
@@ -516,7 +516,7 @@ search_files_cb (GObject *source_object,
 	SearchData *search_data = (SearchData *) user_data;
 	GsShellExtras *self = search_data->self;
 	g_autoptr(GsAppList) list = NULL;
-	GList *l;
+	guint i;
 	GsPluginLoader *plugin_loader = GS_PLUGIN_LOADER (source_object);
 	g_autoptr(GError) error = NULL;
 
@@ -527,13 +527,13 @@ search_files_cb (GObject *source_object,
 			return;
 		}
 		if (g_error_matches (error,
-				     GS_PLUGIN_LOADER_ERROR,
-				     GS_PLUGIN_LOADER_ERROR_NO_RESULTS)) {
+				     GS_PLUGIN_ERROR,
+				     GS_PLUGIN_ERROR_FAILED)) {
 			GsApp *app;
 
 			g_debug ("extras: no search result for %s, showing as missing", search_data->title);
 			app = create_missing_app (search_data);
-			list = g_list_prepend (list, app);
+			gs_app_list_add (list, app);
 		} else {
 			g_autofree gchar *str = NULL;
 
@@ -546,8 +546,8 @@ search_files_cb (GObject *source_object,
 		}
 	}
 
-	for (l = list; l != NULL; l = l->next) {
-		GsApp *app = GS_APP (l->data);
+	for (i = 0; i < gs_app_list_length (list); i++) {
+		GsApp *app = gs_app_list_index (list, i);
 
 		g_debug ("%s\n\n", gs_app_to_string (app));
 		gs_shell_extras_add_app (self, app, search_data);
@@ -578,8 +578,8 @@ file_to_app_cb (GObject *source_object,
 			return;
 		}
 		if (g_error_matches (error,
-				     GS_PLUGIN_LOADER_ERROR,
-				     GS_PLUGIN_LOADER_ERROR_NO_RESULTS)) {
+				     GS_PLUGIN_ERROR,
+				     GS_PLUGIN_ERROR_FAILED)) {
 			g_debug ("extras: no search result for %s, showing as missing", search_data->title);
 			app = create_missing_app (search_data);
 		} else {
@@ -614,7 +614,7 @@ get_search_what_provides_cb (GObject *source_object,
 	SearchData *search_data = (SearchData *) user_data;
 	GsShellExtras *self = search_data->self;
 	g_autoptr(GsAppList) list = NULL;
-	GList *l;
+	guint i;
 	GsPluginLoader *plugin_loader = GS_PLUGIN_LOADER (source_object);
 	g_autoptr(GError) error = NULL;
 
@@ -625,13 +625,13 @@ get_search_what_provides_cb (GObject *source_object,
 			return;
 		}
 		if (g_error_matches (error,
-				     GS_PLUGIN_LOADER_ERROR,
-				     GS_PLUGIN_LOADER_ERROR_NO_RESULTS)) {
+				     GS_PLUGIN_ERROR,
+				     GS_PLUGIN_ERROR_FAILED)) {
 			GsApp *app;
 
 			g_debug ("extras: no search result for %s, showing as missing", search_data->title);
 			app = create_missing_app (search_data);
-			list = g_list_prepend (list, app);
+			gs_app_list_add (list, app);
 		} else {
 			g_autofree gchar *str = NULL;
 
@@ -644,8 +644,8 @@ get_search_what_provides_cb (GObject *source_object,
 		}
 	}
 
-	for (l = list; l != NULL; l = l->next) {
-		GsApp *app = GS_APP (l->data);
+	for (i = 0; i < gs_app_list_length (list); i++) {
+		GsApp *app = gs_app_list_index (list, i);
 
 		g_debug ("%s\n\n", gs_app_to_string (app));
 		gs_shell_extras_add_app (self, app, search_data);
@@ -693,7 +693,7 @@ gs_shell_extras_load (GsShellExtras *self, GPtrArray *array_search_data)
 			g_debug ("searching filename: '%s'", search_data->search_filename);
 			gs_plugin_loader_search_files_async (self->plugin_loader,
 			                                     search_data->search_filename,
-			                                     GS_PLUGIN_REFINE_FLAGS_DEFAULT |
+			                                     GS_PLUGIN_REFINE_FLAGS_REQUIRE_ICON |
 			                                     GS_PLUGIN_REFINE_FLAGS_REQUIRE_RATING |
 			                                     GS_PLUGIN_REFINE_FLAGS_ALLOW_PACKAGES,
 			                                     self->search_cancellable,
@@ -705,7 +705,7 @@ gs_shell_extras_load (GsShellExtras *self, GPtrArray *array_search_data)
 			file = g_file_new_for_path (search_data->package_filename);
 			gs_plugin_loader_file_to_app_async (self->plugin_loader,
 			                                    file,
-			                                    GS_PLUGIN_REFINE_FLAGS_DEFAULT |
+			                                    GS_PLUGIN_REFINE_FLAGS_REQUIRE_ICON |
 			                                    GS_PLUGIN_REFINE_FLAGS_REQUIRE_RATING |
 			                                    GS_PLUGIN_REFINE_FLAGS_ALLOW_PACKAGES,
 			                                    self->search_cancellable,
@@ -715,7 +715,7 @@ gs_shell_extras_load (GsShellExtras *self, GPtrArray *array_search_data)
 			g_debug ("searching what provides: '%s'", search_data->search);
 			gs_plugin_loader_search_what_provides_async (self->plugin_loader,
 			                                             search_data->search,
-			                                             GS_PLUGIN_REFINE_FLAGS_DEFAULT |
+			                                             GS_PLUGIN_REFINE_FLAGS_REQUIRE_ICON |
 			                                             GS_PLUGIN_REFINE_FLAGS_REQUIRE_VERSION |
 			                                             GS_PLUGIN_REFINE_FLAGS_REQUIRE_HISTORY |
 			                                             GS_PLUGIN_REFINE_FLAGS_REQUIRE_SETUP_ACTION |
