@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2013 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2013-2016 Richard Hughes <richard@hughsie.com>
  * Copyright (C) 2013 Matthias Clasen <mclasen@redhat.com>
  *
  * Licensed under the GNU General Public License Version 2
@@ -30,7 +30,7 @@
 #include "gs-review-row.h"
 #include "gs-shell.h"
 #include "gs-shell-moderate.h"
-#include "gs-utils.h"
+#include "gs-common.h"
 
 struct _GsShellModerate
 {
@@ -143,7 +143,7 @@ gs_shell_moderate_get_unvoted_reviews_cb (GObject *source_object,
 					  GAsyncResult *res,
 					  gpointer user_data)
 {
-	GList *l;
+	guint i;
 	GsApp *app;
 	GsShellModerate *self = GS_SHELL_MODERATE (user_data);
 	GsPluginLoader *plugin_loader = GS_PLUGIN_LOADER (source_object);
@@ -157,19 +157,20 @@ gs_shell_moderate_get_unvoted_reviews_cb (GObject *source_object,
 							    res,
 							    &error);
 	if (list == NULL) {
-		if (g_error_matches (error,
-				     GS_PLUGIN_LOADER_ERROR,
-				     GS_PLUGIN_LOADER_ERROR_NO_RESULTS)) {
-			gtk_stack_set_visible_child_name (GTK_STACK (self->stack_install),
-							  "uptodate");
-			return;
-		}
 		if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
 			g_warning ("failed to get moderate apps: %s", error->message);
 		return;
 	}
-	for (l = list; l != NULL; l = l->next) {
-		app = GS_APP (l->data);
+
+	/* no results */
+	if (gs_app_list_length (list) == 0) {
+		gtk_stack_set_visible_child_name (GTK_STACK (self->stack_install),
+						  "uptodate");
+		return;
+	}
+
+	for (i = 0; i < gs_app_list_length (list); i++) {
+		app = gs_app_list_index (list, i);
 		gs_shell_moderate_add_app (self, app);
 	}
 }
@@ -185,7 +186,7 @@ gs_shell_moderate_load (GsShellModerate *self)
 
 	/* get unvoted reviews as apps */
 	gs_plugin_loader_get_unvoted_reviews_async (self->plugin_loader,
-						    GS_PLUGIN_REFINE_FLAGS_DEFAULT |
+						    GS_PLUGIN_REFINE_FLAGS_REQUIRE_ICON |
 						    GS_PLUGIN_REFINE_FLAGS_REQUIRE_SETUP_ACTION |
 						    GS_PLUGIN_REFINE_FLAGS_REQUIRE_VERSION |
 						    GS_PLUGIN_REFINE_FLAGS_REQUIRE_PROVENANCE |
