@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2013 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2013-2016 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -27,7 +27,7 @@
 #include "gs-shell-search.h"
 #include "gs-shell.h"
 #include "gs-app.h"
-#include "gs-utils.h"
+#include "gs-common.h"
 #include "gs-app-row.h"
 
 struct _GsShellSearch
@@ -89,10 +89,10 @@ gs_shell_search_app_row_clicked_cb (GsAppRow *app_row,
  **/
 static void
 gs_shell_search_get_search_cb (GObject *source_object,
-				     GAsyncResult *res,
-				     gpointer user_data)
+			       GAsyncResult *res,
+			       gpointer user_data)
 {
-	GList *l;
+	guint i;
 	GsApp *app;
 	GsShellSearch *self = GS_SHELL_SEARCH (user_data);
 	GsPluginLoader *plugin_loader = GS_PLUGIN_LOADER (source_object);
@@ -106,22 +106,23 @@ gs_shell_search_get_search_cb (GObject *source_object,
 			g_debug ("search cancelled");
 			return;
 		}
-		if (g_error_matches (error,
-				     GS_PLUGIN_LOADER_ERROR,
-				     GS_PLUGIN_LOADER_ERROR_NO_RESULTS)) {
-			g_debug ("no search results to show");
-		} else {
-			g_warning ("failed to get search apps: %s", error->message);
-		}
+		g_warning ("failed to get search apps: %s", error->message);
 		gs_stop_spinner (GTK_SPINNER (self->spinner_search));
 		gtk_stack_set_visible_child_name (GTK_STACK (self->stack_search), "no-results");
 		return;
 	}
 
+	/* no results */
+	if (gs_app_list_length (list) == 0) {
+		g_debug ("no search results to show");
+		gtk_stack_set_visible_child_name (GTK_STACK (self->stack_search), "no-results");
+		return;
+	}
+
 	gs_stop_spinner (GTK_SPINNER (self->spinner_search));
 	gtk_stack_set_visible_child_name (GTK_STACK (self->stack_search), "results");
-	for (l = list; l != NULL; l = l->next) {
-		app = GS_APP (l->data);
+	for (i = 0; i < gs_app_list_length (list); i++) {
+		app = gs_app_list_index (list, i);
 		app_row = gs_app_row_new (app);
 		g_signal_connect (app_row, "button-clicked",
 				  G_CALLBACK (gs_shell_search_app_row_clicked_cb),
@@ -134,7 +135,9 @@ gs_shell_search_get_search_cb (GObject *source_object,
 	}
 
 	if (self->appid_to_show != NULL) {
-		gs_shell_show_details (self->shell, self->appid_to_show);
+		g_autoptr (GsApp) a = NULL;
+		a = gs_app_new (self->appid_to_show);
+		gs_shell_show_app (self->shell, a);
 		g_clear_pointer (&self->appid_to_show, g_free);
 	}
 }
@@ -158,7 +161,7 @@ gs_shell_search_load (GsShellSearch *self)
 	/* search for apps */
 	gs_plugin_loader_search_async (self->plugin_loader,
 				       self->value,
-				       GS_PLUGIN_REFINE_FLAGS_DEFAULT |
+				       GS_PLUGIN_REFINE_FLAGS_REQUIRE_ICON |
 				       GS_PLUGIN_REFINE_FLAGS_REQUIRE_VERSION |
 				       GS_PLUGIN_REFINE_FLAGS_REQUIRE_PROVENANCE |
 				       GS_PLUGIN_REFINE_FLAGS_REQUIRE_HISTORY |
