@@ -84,8 +84,6 @@ void		 gs_shell_show_filename		(GsShell	*shell,
 void		 gs_shell_show_search_result	(GsShell	*shell,
 						 const gchar	*id,
 						 const gchar    *search);
-void		 gs_shell_show_details		(GsShell	*shell,
-						 const gchar	*id);
 void		 gs_shell_show_extras_search	(GsShell	*shell,
 						 const gchar	*mode,
 						 gchar		**resources);
