@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2013 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2013-2016 Richard Hughes <richard@hughsie.com>
  * Copyright (C) 2013 Matthias Clasen <mclasen@redhat.com>
  *
  * Licensed under the GNU General Public License Version 2
@@ -28,7 +28,7 @@
 #include "gs-sources-dialog.h"
 #include "gs-os-release.h"
 #include "gs-sources-dialog-row.h"
-#include "gs-utils.h"
+#include "gs-common.h"
 
 struct _GsSourcesDialog
 {
@@ -72,15 +72,15 @@ add_source (GtkListBox *listbox, GsApp *app)
 	/* split up the types */
 	for (i = 0; i < related->len; i++) {
 		GsApp *app_tmp = g_ptr_array_index (related, i);
-		switch (gs_app_get_id_kind (app_tmp)) {
-		case AS_ID_KIND_WEB_APP:
-		case AS_ID_KIND_DESKTOP:
+		switch (gs_app_get_kind (app_tmp)) {
+		case AS_APP_KIND_WEB_APP:
+		case AS_APP_KIND_DESKTOP:
 			cnt_apps++;
 			break;
-		case AS_ID_KIND_FONT:
-		case AS_ID_KIND_CODEC:
-		case AS_ID_KIND_INPUT_METHOD:
-		case AS_ID_KIND_ADDON:
+		case AS_APP_KIND_FONT:
+		case AS_APP_KIND_CODEC:
+		case AS_APP_KIND_INPUT_METHOD:
+		case AS_APP_KIND_ADDON:
 			cnt_addon++;
 			break;
 		default:
@@ -147,7 +147,7 @@ get_sources_cb (GsPluginLoader *plugin_loader,
 		GAsyncResult *res,
 		GsSourcesDialog *dialog)
 {
-	GList *l;
+	guint i;
 	GsApp *app;
 	g_autoptr(GError) error = NULL;
 	g_autoptr(GsAppList) list = NULL;
@@ -159,12 +159,8 @@ get_sources_cb (GsPluginLoader *plugin_loader,
 	list = gs_plugin_loader_get_sources_finish (plugin_loader, res, &error);
 	if (list == NULL) {
 		if (g_error_matches (error,
-				     GS_PLUGIN_LOADER_ERROR,
-				     GS_PLUGIN_LOADER_ERROR_NO_RESULTS)) {
-			g_debug ("no sources to show");
-		} else if (g_error_matches (error,
-					    G_IO_ERROR,
-					    G_IO_ERROR_CANCELLED)) {
+				     G_IO_ERROR,
+				     G_IO_ERROR_CANCELLED)) {
 			g_debug ("get sources cancelled");
 		} else {
 			g_warning ("failed to get sources: %s", error->message);
@@ -175,13 +171,19 @@ get_sources_cb (GsPluginLoader *plugin_loader,
 		return;
 	}
 
+	/* no results */
+	if (gs_app_list_length (list) == 0) {
+		g_debug ("no sources to show");
+		return;
+	}
+
 	gtk_style_context_remove_class (gtk_widget_get_style_context (dialog->label_header),
 	                                "dim-label");
 
 	/* add each */
 	gtk_stack_set_visible_child_name (GTK_STACK (dialog->stack), "sources");
-	for (l = list; l != NULL; l = l->next) {
-		app = GS_APP (l->data);
+	for (i = 0; i < gs_app_list_length (list); i++) {
+		app = gs_app_list_index (list, i);
 		add_source (GTK_LIST_BOX (dialog->listbox), app);
 	}
 }
@@ -276,8 +278,7 @@ list_row_activated_cb (GtkListBox *list_box,
 	for (i = 0; i < related->len; i++) {
 		GsApp *app_tmp = g_ptr_array_index (related, i);
 		switch (gs_app_get_kind (app_tmp)) {
-		case GS_APP_KIND_NORMAL:
-		case GS_APP_KIND_SYSTEM:
+		case AS_APP_KIND_DESKTOP:
 			add_app (GTK_LIST_BOX (dialog->listbox_apps), app_tmp);
 			cnt_apps++;
 			break;
