@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2013-2014 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2013-2016 Richard Hughes <richard@hughsie.com>
  *
  * Licensed under the GNU General Public License Version 2
  *
@@ -27,7 +27,7 @@
 #include "gs-update-dialog.h"
 #include "gs-app-row.h"
 #include "gs-update-list.h"
-#include "gs-utils.h"
+#include "gs-common.h"
 
 typedef struct {
 	gchar		*title;
@@ -168,7 +168,7 @@ get_installed_updates_cb (GsPluginLoader *plugin_loader,
                           GAsyncResult *res,
                           GsUpdateDialog *dialog)
 {
-	GList *l;
+	guint i;
 	guint64 install_date;
 	g_autoptr(GsAppList) list = NULL;
 	g_autoptr(GError) error = NULL;
@@ -179,14 +179,8 @@ get_installed_updates_cb (GsPluginLoader *plugin_loader,
 	list = gs_plugin_loader_get_updates_finish (plugin_loader, res, &error);
 	if (list == NULL) {
 		if (g_error_matches (error,
-				     GS_PLUGIN_LOADER_ERROR,
-				     GS_PLUGIN_LOADER_ERROR_NO_RESULTS)) {
-			g_debug ("no installed updates to show");
-			gtk_stack_set_visible_child_name (GTK_STACK (dialog->stack), "empty");
-			return;
-		} else if (g_error_matches (error,
-					    G_IO_ERROR,
-					    G_IO_ERROR_CANCELLED)) {
+				    G_IO_ERROR,
+				    G_IO_ERROR_CANCELLED)) {
 			/* This should only ever happen while the dialog is being closed */
 			g_debug ("get installed updates cancelled");
 			return;
@@ -197,8 +191,15 @@ get_installed_updates_cb (GsPluginLoader *plugin_loader,
 		return;
 	}
 
+	/* no results */
+	if (gs_app_list_length (list) == 0) {
+		g_debug ("no installed updates to show");
+		gtk_stack_set_visible_child_name (GTK_STACK (dialog->stack), "empty");
+		return;
+	}
+
 	/* set the header title using any one of the applications */
-	install_date = gs_app_get_install_date (GS_APP (list->data));
+	install_date = gs_app_get_install_date (gs_app_list_index (list, 0));
 	if (install_date > 0) {
 		GtkWidget *header;
 		g_autoptr(GDateTime) date = NULL;
@@ -220,9 +221,9 @@ get_installed_updates_cb (GsPluginLoader *plugin_loader,
 	gtk_stack_set_visible_child_name (GTK_STACK (dialog->stack), "installed-updates-list");
 
 	gs_container_remove_all (GTK_CONTAINER (dialog->list_box_installed_updates));
-	for (l = list; l != NULL; l = l->next) {
+	for (i = 0; i < gs_app_list_length (list); i++) {
 		gs_update_list_add_app (GS_UPDATE_LIST (dialog->list_box_installed_updates),
-					GS_APP (l->data));
+					gs_app_list_index (list, i));
 	}
 }
 
