@@ -108,11 +108,11 @@ notify_offline_update_available (GsUpdateMonitor *monitor)
 static gboolean
 has_important_updates (GsAppList *apps)
 {
-	GList *l;
+	guint i;
 	GsApp *app;
 
-	for (l = apps; l != NULL; l = l->next) {
-		app = GS_APP (l->data);
+	for (i = 0; i < gs_app_list_length (apps); i++) {
+		app = gs_app_list_index (apps, i);
 		if (gs_app_get_update_urgency (app) == AS_URGENCY_KIND_CRITICAL ||
 		    gs_app_get_update_urgency (app) == AS_URGENCY_KIND_HIGH)
 			return TRUE;
@@ -153,7 +153,7 @@ get_updates_finished_cb (GObject *object,
 			 gpointer data)
 {
 	GsUpdateMonitor *monitor = data;
-	GList *l;
+	guint i;
 	GsApp *app;
 	guint64 security_timestamp = 0;
 	guint64 security_timestamp_old = 0;
@@ -174,8 +174,8 @@ get_updates_finished_cb (GObject *object,
 	/* find security updates, or clear timestamp if there are now none */
 	g_settings_get (monitor->settings,
 			"security-timestamp", "x", &security_timestamp_old);
-	for (l = apps; l != NULL; l = l->next) {
-		app = GS_APP (l->data);
+	for (i = 0; i < gs_app_list_length (apps); i++) {
+		app = gs_app_list_index (apps, i);
 		if (gs_app_get_metadata_item (app, "is-security") != NULL) {
 			security_timestamp = g_get_monotonic_time ();
 			break;
@@ -186,7 +186,7 @@ get_updates_finished_cb (GObject *object,
 				"security-timestamp", "x", security_timestamp);
 	}
 
-	g_debug ("Got %d updates", g_list_length (apps));
+	g_debug ("Got %d updates", gs_app_list_length (apps));
 
 	if (has_important_updates (apps) ||
 	    no_updates_for_a_week (monitor)) {
@@ -209,16 +209,18 @@ get_upgrades_finished_cb (GObject *object,
 	/* get result */
 	apps = gs_plugin_loader_get_distro_upgrades_finish (GS_PLUGIN_LOADER (object), res, &error);
 	if (apps == NULL) {
+		if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+			g_warning ("failed to get upgrades: %s",
+				   error->message);
+		}
+		return;
+	}
+
+	/* no results */
+	if (gs_app_list_length (apps) == 0) {
 		g_debug ("no upgrades; withdrawing upgrades-available notification");
 		g_application_withdraw_notification (monitor->application,
 						     "upgrades-available");
-		if (g_error_matches (error,
-		                     GS_PLUGIN_LOADER_ERROR,
-		                     GS_PLUGIN_LOADER_ERROR_NO_RESULTS)) {
-			g_debug ("no upgrades to show");
-		} else if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
-			g_warning ("failed to get upgrades: %s", error->message);
-		}
 		return;
 	}
 
@@ -227,7 +229,7 @@ get_upgrades_finished_cb (GObject *object,
 		return;
 
 	/* just get the first result : FIXME, do we sort these by date? */
-	app = GS_APP (apps->data);
+	app = gs_app_list_index (apps, 0);
 
 	/* TRANSLATORS: this is a distro upgrade, the replacement would be the
 	 * distro name, e.g. 'Fedora' */
@@ -416,33 +418,32 @@ get_updates_historical_cb (GObject *object, GAsyncResult *res, gpointer data)
 	apps = gs_plugin_loader_get_updates_finish (GS_PLUGIN_LOADER (object), res, &error);
 	if (apps == NULL) {
 
-		/* nothing has been updated offline */
-		if (g_error_matches (error,
-				     GS_PLUGIN_LOADER_ERROR,
-				     GS_PLUGIN_LOADER_ERROR_NO_RESULTS)) {
-			g_debug ("no historical updates; withdrawing notification");
-			g_application_withdraw_notification (monitor->application,
-							     "updates-available");
-		} else {
-			/* save this in case the user clicks the
-			 * 'Show Details' button from the notification below */
-			g_clear_error (&monitor->last_offline_error);
-			monitor->last_offline_error = g_error_copy (error);
+		/* save this in case the user clicks the
+		 * 'Show Details' button from the notification below */
+		g_clear_error (&monitor->last_offline_error);
+		monitor->last_offline_error = g_error_copy (error);
 
-			/* TRANSLATORS: title when we offline updates have failed */
-			notification = g_notification_new (_("Software Updates Failed"));
-			/* TRANSLATORS: message when we offline updates have failed */
-			g_notification_set_body (notification, _("An important OS update failed to be installed."));
-			g_application_send_notification (monitor->application, "offline-updates", notification);
-			g_notification_add_button (notification, _("Show Details"), "app.show-offline-update-error");
-			g_notification_set_default_action (notification, "app.show-offline-update-error");
-			g_application_send_notification (monitor->application, "offline-updates", notification);
-		}
+		/* TRANSLATORS: title when we offline updates have failed */
+		notification = g_notification_new (_("Software Updates Failed"));
+		/* TRANSLATORS: message when we offline updates have failed */
+		g_notification_set_body (notification, _("An important OS update failed to be installed."));
+		g_application_send_notification (monitor->application, "offline-updates", notification);
+		g_notification_add_button (notification, _("Show Details"), "app.show-offline-update-error");
+		g_notification_set_default_action (notification, "app.show-offline-update-error");
+		g_application_send_notification (monitor->application, "offline-updates", notification);
+		return;
+	}
+
+	/* no results */
+	if (gs_app_list_length (apps) == 0) {
+		g_debug ("no historical updates; withdrawing notification");
+		g_application_withdraw_notification (monitor->application,
+						     "updates-available");
 		return;
 	}
 
 	/* have we notified about this before */
-	app = GS_APP (apps->data);
+	app = gs_app_list_index (apps, 0);
 	g_settings_get (monitor->settings,
 			"install-timestamp", "x", &time_last_notified);
 	if (time_last_notified >= gs_app_get_install_date (app))
@@ -451,11 +452,11 @@ get_updates_historical_cb (GObject *object, GAsyncResult *res, gpointer data)
 	/* TRANSLATORS: title when we've done offline updates */
 	title = ngettext ("Software Update Installed",
 			  "Software Updates Installed",
-			  g_list_length (apps));
+			  gs_app_list_length (apps));
 	/* TRANSLATORS: message when we've done offline updates */
 	message = ngettext ("An important OS update has been installed.",
 			    "Important OS updates have been installed.",
-			    g_list_length (apps));
+			    gs_app_list_length (apps));
 
 	notification = g_notification_new (title);
 	g_notification_set_body (notification, message);
