@@ -21,8 +21,7 @@
 
 #include "config.h"
 
-#include <gs-plugin.h>
-#include <gs-utils.h>
+#include <gnome-software.h>
 
 #include "gs-appstream.h"
 
@@ -35,50 +34,31 @@ static void
 gs_refine_item_pixbuf (GsPlugin *plugin, GsApp *app, AsApp *item)
 {
 	AsIcon *icon;
-	gboolean ret;
 	g_autoptr(GError) error = NULL;
 	g_autofree gchar *fn = NULL;
 	g_autofree gchar *cachedir = NULL;
 
 	icon = as_app_get_icon_default (item);
+	gs_app_set_icon (app, icon);
 	switch (as_icon_get_kind (icon)) {
 	case AS_ICON_KIND_REMOTE:
-		gs_app_set_icon (app, icon);
 		if (as_icon_get_filename (icon) == NULL) {
-			cachedir = gs_utils_get_cachedir ("icons", NULL);
-			fn = g_build_filename (cachedir, as_icon_get_name (icon), NULL);
+			fn = gs_utils_get_cache_filename ("icons",
+							  as_icon_get_name (icon),
+							  GS_UTILS_CACHE_FLAG_WRITEABLE,
+							  NULL);
 			as_icon_set_filename (icon, fn);
+			cachedir = g_path_get_basename (fn);
 			as_icon_set_prefix (icon, cachedir);
 		}
-		if (g_file_test (fn, G_FILE_TEST_EXISTS)) {
-			as_icon_set_kind (icon, AS_ICON_KIND_LOCAL);
-			ret = gs_app_load_icon (app, gs_plugin_get_scale (plugin), &error);
-			if (!ret) {
-				g_warning ("failed to load icon %s: %s",
-					   as_icon_get_name (icon),
-					   error->message);
-				return;
-			}
-		}
 		break;
 	case AS_ICON_KIND_STOCK:
+		break;
 	case AS_ICON_KIND_LOCAL:
-		gs_app_set_icon (app, icon);
-
 		/* does not exist, so try to find using the icon theme */
 		if (as_icon_get_kind (icon) == AS_ICON_KIND_LOCAL &&
 		    as_icon_get_filename (icon) == NULL)
 			as_icon_set_kind (icon, AS_ICON_KIND_STOCK);
-
-		/* load */
-		ret = gs_app_load_icon (app, gs_plugin_get_scale (plugin), &error);
-		if (!ret) {
-			g_warning ("failed to load %s icon %s: %s",
-				   as_icon_kind_to_string (as_icon_get_kind (icon)),
-				   as_icon_get_name (icon),
-				   error->message);
-				return;
-		}
 		break;
 	case AS_ICON_KIND_CACHED:
 		if (gs_plugin_get_scale (plugin) == 2)
@@ -87,7 +67,7 @@ gs_refine_item_pixbuf (GsPlugin *plugin, GsApp *app, AsApp *item)
 			icon = as_app_get_icon_for_size (item, 64, 64);
 		if (icon == NULL) {
 			g_warning ("failed to find cached icon %s",
-				   as_icon_get_name (icon));
+				   as_app_get_id (item));
 			return;
 		}
 		if (!as_icon_load (icon, AS_ICON_LOAD_FLAG_SEARCH_SIZE, &error)) {
@@ -313,11 +293,14 @@ gs_refine_item_management_plugin (GsApp *app, AsApp *item)
 	bundles = as_app_get_bundles (item);
 	for (i = 0; i < bundles->len; i++) {
 		AsBundle *bundle = g_ptr_array_index (bundles, i);
-		if (as_bundle_get_kind (bundle) == AS_BUNDLE_KIND_XDG_APP) {
-			gs_app_set_management_plugin (app, "xdg-app");
-			gs_app_add_source (app, as_bundle_get_id (bundle));
+		AsBundleKind kind = as_bundle_get_kind (bundle);
 
-			/* automatically add runtime */
+		/* common to all bundle formats */
+		gs_app_set_management_plugin (app, as_bundle_kind_to_string (kind));
+		gs_app_add_source (app, as_bundle_get_id (bundle));
+
+		/* automatically add runtime */
+		if (kind == AS_BUNDLE_KIND_XDG_APP) {
 			runtime = as_bundle_get_runtime (bundle);
 			if (runtime != NULL) {
 				g_autoptr(GsApp) app2 = NULL;
@@ -330,11 +313,6 @@ gs_refine_item_management_plugin (GsApp *app, AsApp *item)
 			}
 			break;
 		}
-		if (as_bundle_get_kind (bundle) == AS_BUNDLE_KIND_LIMBA) {
-			gs_app_set_management_plugin (app, "limba");
-			gs_app_add_source (app, as_bundle_get_id (bundle));
-			break;
-		}
 	}
 }
 
@@ -460,6 +438,25 @@ gs_appstream_refine_app (GsPlugin *plugin,
 		}
 	}
 
+	/* types we can never launch */
+	switch (gs_app_get_kind (app)) {
+	case AS_APP_KIND_ADDON:
+	case AS_APP_KIND_CODEC:
+	case AS_APP_KIND_FIRMWARE:
+	case AS_APP_KIND_FONT:
+	case AS_APP_KIND_GENERIC:
+	case AS_APP_KIND_INPUT_METHOD:
+	case AS_APP_KIND_LOCALIZATION:
+	case AS_APP_KIND_OS_UPDATE:
+	case AS_APP_KIND_OS_UPGRADE:
+	case AS_APP_KIND_RUNTIME:
+	case AS_APP_KIND_SOURCE:
+		gs_app_add_quirk (app, AS_APP_QUIRK_NOT_LAUNCHABLE);
+		break;
+	default:
+		break;
+	}
+
 	/* set management plugin automatically */
 	gs_refine_item_management_plugin (app, item);
 
