@@ -20,12 +20,8 @@
  */
 
 #include <config.h>
-#include <glib/gi18n.h>
-#include <appstream-glib.h>
 
-#include <gs-utils.h>
-#include <gs-plugin.h>
-#include <gs-plugin-loader.h>
+#include <gnome-software.h>
 
 #include "gs-appstream.h"
 
@@ -69,19 +65,10 @@ gs_plugin_initialize (GsPlugin *plugin)
 	as_store_set_watch_flags (priv->store,
 				  AS_STORE_WATCH_FLAG_ADDED |
 				  AS_STORE_WATCH_FLAG_REMOVED);
-}
 
-/**
- * gs_plugin_order_after:
- */
-const gchar **
-gs_plugin_order_after (GsPlugin *plugin)
-{
-	static const gchar *deps[] = {
-		"menu-spec-categories",	/* need category list */
-		"dpkg",			/* need package name */
-		NULL };
-	return deps;
+	/* need category list and package name */
+	gs_plugin_add_rule (plugin, GS_PLUGIN_RULE_RUN_AFTER, "menu-spec-categories");
+	gs_plugin_add_rule (plugin, GS_PLUGIN_RULE_RUN_AFTER, "dpkg");
 }
 
 /**
@@ -152,7 +139,8 @@ gs_plugin_setup (GsPlugin *plugin, GCancellable *cancellable, GError **error)
 	GPtrArray *items;
 	gboolean ret;
 	const gchar *origin;
-	const gchar *tmp;
+	const gchar *test_xml;
+	const gchar *test_icon_root;
 	guint *perc;
 	guint i;
 	g_autoptr(GHashTable) origins = NULL;
@@ -164,10 +152,12 @@ gs_plugin_setup (GsPlugin *plugin, GCancellable *cancellable, GError **error)
 	}
 
 	/* only when in self test */
-	tmp = g_getenv ("GS_SELF_TEST_APPSTREAM_XML");
-	if (tmp != NULL) {
-		g_debug ("using self test data of %s", tmp);
-		if (!as_store_from_xml (priv->store, tmp, NULL, error))
+	test_xml = g_getenv ("GS_SELF_TEST_APPSTREAM_XML");
+	if (test_xml != NULL) {
+		test_icon_root = g_getenv ("GS_SELF_TEST_APPSTREAM_ICON_ROOT");
+		g_debug ("using self test data of %s... with icon root %s",
+			 test_xml, test_icon_root);
+		if (!as_store_from_xml (priv->store, test_xml, test_icon_root, error))
 			return FALSE;
 	} else {
 		ret = as_store_load (priv->store,
@@ -175,7 +165,8 @@ gs_plugin_setup (GsPlugin *plugin, GCancellable *cancellable, GError **error)
 				     AS_STORE_LOAD_FLAG_APP_INFO_USER |
 				     AS_STORE_LOAD_FLAG_APPDATA |
 				     AS_STORE_LOAD_FLAG_DESKTOP |
-				     AS_STORE_LOAD_FLAG_XDG_APP_USER |
+				     AS_STORE_LOAD_FLAG_FLATPAK_USER |
+				     AS_STORE_LOAD_FLAG_FLATPAK_SYSTEM |
 				     AS_STORE_LOAD_FLAG_APP_INSTALL,
 				     NULL,
 				     error);
@@ -186,8 +177,8 @@ gs_plugin_setup (GsPlugin *plugin, GCancellable *cancellable, GError **error)
 	if (items->len == 0) {
 		g_warning ("No AppStream data, try 'make install-sample-data' in data/");
 		g_set_error (error,
-			     GS_PLUGIN_LOADER_ERROR,
-			     GS_PLUGIN_LOADER_ERROR_FAILED,
+			     GS_PLUGIN_ERROR,
+			     GS_PLUGIN_ERROR_FAILED,
 			     "No AppStream data found");
 		return FALSE;
 	}
@@ -247,6 +238,7 @@ gs_plugin_refine_from_id (GsPlugin *plugin,
 {
 	GsPluginData *priv = gs_plugin_get_data (plugin);
 	const gchar *id;
+	guint i;
 	AsApp *item = NULL;
 
 	/* unfound */
@@ -256,13 +248,35 @@ gs_plugin_refine_from_id (GsPlugin *plugin,
 	id = gs_app_get_id (app);
 	if (id == NULL)
 		return TRUE;
-	item = as_store_get_app_by_id (priv->store, id);
-	if (item == NULL &&
-	    gs_app_has_quirk (app, AS_APP_QUIRK_MATCH_ANY_PREFIX)) {
-		item = as_store_get_app_by_id_ignore_prefix (priv->store, id);
-		if (item != NULL)
-			g_debug ("found %s for wildcard %s", as_app_get_id (item), id);
+
+	/* find the best app when matching any prefixes */
+	if (gs_app_has_quirk (app, AS_APP_QUIRK_MATCH_ANY_PREFIX)) {
+		g_autoptr(GPtrArray) items = NULL;
+		items = as_store_get_apps_by_id (priv->store, id);
+		for (i = 0; i < items->len; i++) {
+			AsApp *item_tmp = NULL;
+
+			/* does the app have an installation method */
+			item_tmp = g_ptr_array_index (items, i);
+			if (as_app_get_pkgname_default (item_tmp) == NULL &&
+			    as_app_get_bundle_default (item_tmp) == NULL) {
+				g_debug ("not using %s for wildcard as "
+					 "no bundle or pkgname",
+					 as_app_get_id (item_tmp));
+				continue;
+			}
+
+			/* we could match more than one, so list all and return
+			 * the last entry -- fingers crossed it's only one... */
+			g_debug ("found %s for wildcard %s",
+				 as_app_get_id (item_tmp), id);
+			item = item_tmp;
+		}
+	} else {
+		item = as_store_get_app_by_id (priv->store, id);
 	}
+
+	/* nothing found */
 	if (item == NULL)
 		return TRUE;
 
@@ -311,7 +325,7 @@ gs_plugin_refine_from_pkgname (GsPlugin *plugin,
  */
 gboolean
 gs_plugin_add_distro_upgrades (GsPlugin *plugin,
-			       GList **list,
+			       GsAppList *list,
 			       GCancellable *cancellable,
 			       GError **error)
 {
@@ -369,7 +383,7 @@ gs_plugin_refine_app (GsPlugin *plugin,
 gboolean
 gs_plugin_add_category_apps (GsPlugin *plugin,
 			     GsCategory *category,
-			     GList **list,
+			     GsAppList *list,
 			     GCancellable *cancellable,
 			     GError **error)
 {
@@ -423,7 +437,7 @@ gs_plugin_add_category_apps (GsPlugin *plugin,
  */
 static gboolean
 gs_plugin_add_search_item (GsPlugin *plugin,
-			   GList **list,
+			   GsAppList *list,
 			   AsApp *item,
 			   gchar **values,
 			   GCancellable *cancellable,
@@ -462,7 +476,7 @@ gs_plugin_add_search_item (GsPlugin *plugin,
 gboolean
 gs_plugin_add_search (GsPlugin *plugin,
 		      gchar **values,
-		      GList **list,
+		      GsAppList *list,
 		      GCancellable *cancellable,
 		      GError **error)
 {
@@ -494,7 +508,7 @@ gs_plugin_add_search (GsPlugin *plugin,
  */
 gboolean
 gs_plugin_add_installed (GsPlugin *plugin,
-			 GList **list,
+			 GsAppList *list,
 			 GCancellable *cancellable,
 			 GError **error)
 {
@@ -525,27 +539,26 @@ gs_plugin_add_installed (GsPlugin *plugin,
  * gs_plugin_add_categories_for_app:
  */
 static void
-gs_plugin_add_categories_for_app (GList *list, AsApp *app)
+gs_plugin_add_categories_for_app (GPtrArray *list, AsApp *app)
 {
-	GList *l;
-	GList *l2;
+	guint i, j;
 	GsCategory *category;
 	GsCategory *parent;
 	gboolean found_subcat;
 
 	/* does it match the main category */
-	for (l = list; l != NULL; l = l->next) {
-		g_autoptr(GList) children = NULL;
-		parent = GS_CATEGORY (l->data);
+	for (i = 0; i < list->len; i++) {
+		GPtrArray *children;
+		parent = GS_CATEGORY (g_ptr_array_index (list, i));
 		if (!as_app_has_category (app, gs_category_get_id (parent)))
 			continue;
 		gs_category_increment_size (parent);
 
 		/* does it match any sub-categories */
 		found_subcat = FALSE;
-		children = gs_category_get_subcategories (parent);
-		for (l2 = children; l2 != NULL; l2 = l2->next) {
-			category = GS_CATEGORY (l2->data);
+		children = gs_category_get_children (parent);
+		for (j = 0; j < children->len; j++) {
+			category = GS_CATEGORY (g_ptr_array_index (children, j));
 			if (!as_app_has_category (app, gs_category_get_id (category)))
 				continue;
 			gs_category_increment_size (category);
@@ -557,8 +570,8 @@ gs_plugin_add_categories_for_app (GList *list, AsApp *app)
 		if (!found_subcat) {
 			category = gs_category_find_child (parent, "other");
 			if (category == NULL) {
-				category = gs_category_new (parent, "other", NULL);
-				gs_category_add_subcategory (parent, category);
+				category = gs_category_new ("other", NULL);
+				gs_category_add_child (parent, category);
 				g_object_unref (category);
 			}
 			as_app_add_category (app, gs_category_get_id (category));
@@ -572,7 +585,7 @@ gs_plugin_add_categories_for_app (GList *list, AsApp *app)
  */
 gboolean
 gs_plugin_add_categories (GsPlugin *plugin,
-			  GList **list,
+			  GPtrArray *list,
 			  GCancellable *cancellable,
 			  GError **error)
 {
@@ -592,7 +605,7 @@ gs_plugin_add_categories (GsPlugin *plugin,
 			continue;
 		if (as_app_get_priority (app) < 0)
 			continue;
-		gs_plugin_add_categories_for_app (*list, app);
+		gs_plugin_add_categories_for_app (list, app);
 	}
 	return TRUE;
 }
@@ -602,7 +615,7 @@ gs_plugin_add_categories (GsPlugin *plugin,
  */
 gboolean
 gs_plugin_add_popular (GsPlugin *plugin,
-		       GList **list,
+		       GsAppList *list,
 		       GCancellable *cancellable,
 		       GError **error)
 {
@@ -635,7 +648,7 @@ gs_plugin_add_popular (GsPlugin *plugin,
  */
 gboolean
 gs_plugin_add_featured (GsPlugin *plugin,
-			GList **list,
+			GsAppList *list,
 			GCancellable *cancellable,
 			GError **error)
 {
