@@ -21,7 +21,7 @@
 
 #include <config.h>
 
-#include <gs-plugin.h>
+#include <gnome-software.h>
 
 /*
  * SECTION:
@@ -32,17 +32,6 @@ struct GsPluginData {
 	guint			 quirk_id;
 };
 
-/**
- * gs_plugin_order_after:
- */
-const gchar **
-gs_plugin_order_after (GsPlugin *plugin)
-{
-	static const gchar *deps[] = { "appstream",
-				       NULL };
-	return deps;
-}
-
 /**
  * gs_plugin_initialize:
  */
@@ -55,6 +44,9 @@ gs_plugin_initialize (GsPlugin *plugin)
 			 gs_plugin_get_name (plugin));
 		gs_plugin_set_enabled (plugin, FALSE);
 	}
+
+	/* need help from appstream */
+	gs_plugin_add_rule (plugin, GS_PLUGIN_RULE_RUN_AFTER, "appstream");
 }
 
 /**
@@ -147,7 +139,7 @@ gs_plugin_dummy_poll_cb (gpointer user_data)
 gboolean
 gs_plugin_add_search (GsPlugin *plugin,
 		      gchar **values,
-		      GList **list,
+		      GsAppList *list,
 		      GCancellable *cancellable,
 		      GError **error)
 {
@@ -199,7 +191,7 @@ gs_plugin_add_search (GsPlugin *plugin,
  */
 gboolean
 gs_plugin_add_updates (GsPlugin *plugin,
-		       GList **list,
+		       GsAppList *list,
 		       GCancellable *cancellable,
 		       GError **error)
 {
@@ -267,7 +259,7 @@ gs_plugin_add_updates (GsPlugin *plugin,
  */
 gboolean
 gs_plugin_add_installed (GsPlugin *plugin,
-			 GList **list,
+			 GsAppList *list,
 			 GCancellable *cancellable,
 			 GError **error)
 {
@@ -394,7 +386,7 @@ gs_plugin_refine_app (GsPlugin *plugin,
 	if (flags & GS_PLUGIN_REFINE_FLAGS_REQUIRE_LICENSE) {
 		if (g_strcmp0 (gs_app_get_id (app), "chiron.desktop") == 0 ||
 		    g_strcmp0 (gs_app_get_id (app), "zeus.desktop") == 0)
-			gs_app_set_license (app, GS_APP_QUALITY_NORMAL, "GPL-2.0+");
+			gs_app_set_license (app, GS_APP_QUALITY_HIGHEST, "GPL-2.0+");
 	}
 
 	/* homepage */
@@ -405,6 +397,12 @@ gs_plugin_refine_app (GsPlugin *plugin,
 		}
 	}
 
+	/* origin */
+	if (flags & GS_PLUGIN_REFINE_FLAGS_REQUIRE_ORIGIN) {
+		if (g_strcmp0 (gs_app_get_id (app), "zeus-spell.addon") == 0)
+			gs_app_set_origin (app, "london-east");
+	}
+
 	/* description */
 	if (flags & GS_PLUGIN_REFINE_FLAGS_REQUIRE_DESCRIPTION) {
 		if (g_strcmp0 (gs_app_get_id (app), "chiron.desktop") == 0) {
@@ -466,7 +464,7 @@ gs_plugin_refine_app (GsPlugin *plugin,
 gboolean
 gs_plugin_add_category_apps (GsPlugin *plugin,
 			     GsCategory *category,
-			     GList **list,
+			     GsAppList *list,
 			     GCancellable *cancellable,
 			     GError **error)
 {
@@ -488,7 +486,7 @@ gs_plugin_add_category_apps (GsPlugin *plugin,
  */
 gboolean
 gs_plugin_add_distro_upgrades (GsPlugin *plugin,
-			       GList **list,
+			       GsAppList *list,
 			       GCancellable *cancellable,
 			       GError **error)
 {
