@@ -24,11 +24,7 @@
 #define _GNU_SOURCE
 #include <string.h>
 
-#include <glib/gi18n.h>
-#include <libsoup/soup.h>
-
-#include <gs-plugin.h>
-#include <gs-utils.h>
+#include <gnome-software.h>
 
 /*
  * SECTION:
@@ -37,18 +33,6 @@
  * If the epiphany binary is not present then it self-disables.
  */
 
-/**
- * gs_plugin_order_after:
- */
-const gchar **
-gs_plugin_order_after (GsPlugin *plugin)
-{
-	static const gchar *deps[] = {
-		"appstream",
-		NULL };
-	return deps;
-}
-
 /**
  * gs_plugin_initialize:
  */
@@ -64,6 +48,9 @@ gs_plugin_initialize (GsPlugin *plugin)
 		g_debug ("disabling '%s' as epiphany does not exist",
 			 gs_plugin_get_name (plugin));
 	}
+
+	/* need help from appstream */
+	gs_plugin_add_rule (plugin, GS_PLUGIN_RULE_RUN_AFTER, "appstream");
 }
 
 /**
