@@ -22,10 +22,7 @@
 #include <config.h>
 
 #include <json-glib/json-glib.h>
-
-#include <gs-plugin.h>
-#include <gs-os-release.h>
-#include <gs-utils.h>
+#include <gnome-software.h>
 
 #define FEDORA_PKGDB_COLLECTIONS_API_URI "https://admin.fedoraproject.org/pkgdb/api/collections/"
 
@@ -97,16 +94,18 @@ gboolean
 gs_plugin_setup (GsPlugin *plugin, GCancellable *cancellable, GError **error)
 {
 	GsPluginData *priv = gs_plugin_get_data (plugin);
+	const gchar *verstr = NULL;
 	gchar *endptr = NULL;
-	g_autofree gchar *cachedir = NULL;
-	g_autofree gchar *verstr = NULL;
 	g_autoptr(GFile) file = NULL;
+	g_autoptr(GsOsRelease) os_release = NULL;
 
-	/* create the cachedir */
-	cachedir = gs_utils_get_cachedir ("upgrades", error);
-	if (cachedir == NULL)
+	/* get the file to cache */
+	priv->cachefn = gs_utils_get_cache_filename ("upgrades",
+						     "fedora.json",
+						     GS_UTILS_CACHE_FLAG_WRITEABLE,
+						     error);
+	if (priv->cachefn == NULL)
 		return FALSE;
-	priv->cachefn = g_build_filename (cachedir, "fedora.json", NULL);
 
 	/* watch this in case it is changed by the user */
 	file = g_file_new_for_path (priv->cachefn);
@@ -120,10 +119,13 @@ gs_plugin_setup (GsPlugin *plugin, GCancellable *cancellable, GError **error)
 			  G_CALLBACK (gs_plugin_fedora_distro_upgrades_changed_cb), plugin);
 
 	/* read os-release for the current versions */
-	priv->os_name = gs_os_release_get_name (error);
+	os_release = gs_os_release_new (error);
+	if (os_release == NULL)
+		return FALSE;
+	priv->os_name = g_strdup (gs_os_release_get_name (os_release));
 	if (priv->os_name == NULL)
 		return FALSE;
-	verstr = gs_os_release_get_version_id (error);
+	verstr = gs_os_release_get_version_id (os_release);
 	if (verstr == NULL)
 		return FALSE;
 
@@ -302,7 +304,7 @@ parse_pkgdb_collections_data (const gchar *data,
  */
 gboolean
 gs_plugin_add_distro_upgrades (GsPlugin *plugin,
-			       GList **list,
+			       GsAppList *list,
 			       GCancellable *cancellable,
 			       GError **error)
 {
