@@ -22,15 +22,12 @@
 #include <config.h>
 
 #include <fwupd.h>
-#include <appstream-glib.h>
 #include <fcntl.h>
 #include <gio/gio.h>
 #include <gio/gunixfdlist.h>
 #include <glib/gstdio.h>
 
-#include <gs-plugin.h>
-
-#include "gs-utils.h"
+#include <gnome-software.h>
 
 /*
  * SECTION:
@@ -44,7 +41,6 @@ struct GsPluginData {
 	FwupdClient		*client;
 	GPtrArray		*to_download;
 	GPtrArray		*to_ignore;
-	gchar			*cachedir;
 	gchar			*lvfs_sig_fn;
 	gchar			*lvfs_sig_hash;
 	gchar			*config_fn;
@@ -78,7 +74,6 @@ void
 gs_plugin_destroy (GsPlugin *plugin)
 {
 	GsPluginData *priv = gs_plugin_get_data (plugin);
-	g_free (priv->cachedir);
 	g_free (priv->lvfs_sig_fn);
 	g_free (priv->lvfs_sig_hash);
 	g_free (priv->config_fn);
@@ -103,8 +98,36 @@ gs_plugin_adopt_app (GsPlugin *plugin, GsApp *app)
 static void
 gs_plugin_fwupd_changed_cb (FwupdClient *client, GsPlugin *plugin)
 {
+#if !FWUPD_CHECK_VERSION(0,7,1)
+	/* fwupd < 0.7.1 only supported the ::Changed() signal */
+	gs_plugin_updates_changed (plugin);
+#endif
+}
+
+#if FWUPD_CHECK_VERSION(0,7,1)
+/**
+ * gs_plugin_fwupd_device_changed_cb:
+ */
+static void
+gs_plugin_fwupd_device_changed_cb (FwupdClient *client,
+				   FwupdResult *device,
+				   GsPlugin *plugin)
+{
+	/* fwupd >= 0.7.1 supports per-device signals, and also the
+	 * SUPPORTED flag -- so we can limit number of UI refreshes */
+	if (!fwupd_result_has_device_flag (device, FU_DEVICE_FLAG_SUPPORTED)) {
+		g_debug ("%s changed (not supported) so ignoring",
+			 fwupd_result_get_device_id (device));
+		return;
+	}
+
+	/* If the flag is set the device matches something in the
+	 * metadata as therefor is worth refreshing the update list */
+	g_debug ("%s changed (supported) so reloading",
+		 fwupd_result_get_device_id (device));
 	gs_plugin_updates_changed (plugin);
 }
+#endif
 
 /**
  * gs_plugin_setup:
@@ -116,24 +139,26 @@ gs_plugin_setup (GsPlugin *plugin, GCancellable *cancellable, GError **error)
 	gsize len;
 	g_autofree gchar *data = NULL;
 
-	/* already done */
-	if (priv->cachedir != NULL)
-		return TRUE;
-
 	/* register D-Bus errors */
 	fwupd_error_quark ();
 	g_signal_connect (priv->client, "changed",
 			  G_CALLBACK (gs_plugin_fwupd_changed_cb), plugin);
-
-	/* create the cache location */
-	priv->cachedir = gs_utils_get_cachedir ("firmware", error);
-	if (priv->cachedir == NULL)
-		return FALSE;
+#if FWUPD_CHECK_VERSION(0,7,1)
+	g_signal_connect (priv->client, "device-added",
+			  G_CALLBACK (gs_plugin_fwupd_device_changed_cb), plugin);
+	g_signal_connect (priv->client, "device-removed",
+			  G_CALLBACK (gs_plugin_fwupd_device_changed_cb), plugin);
+	g_signal_connect (priv->client, "device-changed",
+			  G_CALLBACK (gs_plugin_fwupd_device_changed_cb), plugin);
+#endif
 
 	/* get the hash of the previously downloaded file */
-	priv->lvfs_sig_fn = g_build_filename (priv->cachedir,
-						      "firmware.xml.gz.asc",
-						      NULL);
+	priv->lvfs_sig_fn = gs_utils_get_cache_filename ("firmware",
+							 "firmware.xml.gz.asc",
+							 GS_UTILS_CACHE_FLAG_WRITEABLE,
+							 error);
+	if (priv->lvfs_sig_fn == NULL)
+		return FALSE;
 	if (g_file_test (priv->lvfs_sig_fn, G_FILE_TEST_EXISTS)) {
 		if (!g_file_get_contents (priv->lvfs_sig_fn,
 					  &data, &len, error))
@@ -196,6 +221,7 @@ gs_plugin_fwupd_new_app_from_results (FwupdResult *res)
 	/* default stuff */
 	app = gs_app_new (fwupd_result_get_update_id (res));
 	gs_app_set_kind (app, AS_APP_KIND_FIRMWARE);
+	gs_app_add_quirk (app, AS_APP_QUIRK_NOT_LAUNCHABLE);
 	gs_app_set_management_plugin (app, "fwupd");
 	gs_app_add_category (app, "System");
 	gs_app_set_metadata (app, "fwupd::DeviceID",
@@ -232,6 +258,10 @@ gs_plugin_fwupd_new_app_from_results (FwupdResult *res)
 		gs_app_set_summary (app, GS_APP_QUALITY_NORMAL,
 				    fwupd_result_get_update_summary (res));
 	}
+	if (fwupd_result_get_update_homepage (res) != NULL) {
+		gs_app_set_url (app, AS_URL_KIND_HOMEPAGE,
+				fwupd_result_get_update_homepage (res));
+	}
 	if (fwupd_result_get_device_version (res) != NULL) {
 		gs_app_set_version (app, fwupd_result_get_device_version (res));
 	}
@@ -249,6 +279,13 @@ gs_plugin_fwupd_new_app_from_results (FwupdResult *res)
 		gs_app_set_license (app, GS_APP_QUALITY_NORMAL,
 				    fwupd_result_get_update_license (res));
 	}
+	if (fwupd_result_get_device_description (res) != NULL) {
+		g_autofree gchar *tmp = NULL;
+		tmp = as_markup_convert (fwupd_result_get_device_description (res),
+					 AS_MARKUP_CONVERT_FORMAT_SIMPLE, NULL);
+		if (tmp != NULL)
+			gs_app_set_description (app, GS_APP_QUALITY_NORMAL, tmp);
+	}
 	if (fwupd_result_get_update_description (res) != NULL) {
 		g_autofree gchar *tmp = NULL;
 		tmp = as_markup_convert (fwupd_result_get_update_description (res),
@@ -271,11 +308,10 @@ gs_plugin_fwupd_new_app_from_results (FwupdResult *res)
  */
 static gboolean
 gs_plugin_add_update_app (GsPlugin *plugin,
-			  GList **list,
+			  GsAppList *list,
 			  FwupdResult *res,
 			  GError **error)
 {
-	GsPluginData *priv = gs_plugin_get_data (plugin);
 	FwupdDeviceFlags flags = 0;
 	const gchar *update_hash;
 	const gchar *update_uri;
@@ -338,8 +374,12 @@ gs_plugin_add_update_app (GsPlugin *plugin,
 
 		/* does the firmware already exist in the cache? */
 		basename = g_path_get_basename (update_uri);
-		filename_cache = g_build_filename (priv->cachedir,
-						   basename, NULL);
+		filename_cache = gs_utils_get_cache_filename ("firmware",
+							      basename,
+							      GS_UTILS_CACHE_FLAG_NONE,
+							      error);
+		if (filename_cache == NULL)
+			return FALSE;
 		if (!g_file_test (filename_cache, G_FILE_TEST_EXISTS)) {
 			gs_plugin_fwupd_add_required_location (plugin, update_uri);
 			g_set_error (error,
@@ -380,7 +420,7 @@ gs_plugin_add_update_app (GsPlugin *plugin,
  */
 gboolean
 gs_plugin_add_updates_historical (GsPlugin *plugin,
-				  GList **list,
+				  GsAppList *list,
 				  GCancellable *cancellable,
 				  GError **error)
 {
@@ -421,7 +461,7 @@ gs_plugin_add_updates_historical (GsPlugin *plugin,
  */
 gboolean
 gs_plugin_add_updates (GsPlugin *plugin,
-		       GList **list,
+		       GsAppList *list,
 		       GCancellable *cancellable,
 		       GError **error)
 {
@@ -538,7 +578,12 @@ gs_plugin_fwupd_check_lvfs_metadata (GsPlugin *plugin,
 
 	/* download the payload and save to file */
 	basename_data = g_path_get_basename (url_data);
-	cache_fn_data = g_build_filename (priv->cachedir, basename_data, NULL);
+	cache_fn_data = gs_utils_get_cache_filename ("firmware",
+						     basename_data,
+						     GS_UTILS_CACHE_FLAG_WRITEABLE,
+						     error);
+	if (cache_fn_data == NULL)
+		return FALSE;
 	g_debug ("saving new LVFS data to %s:", cache_fn_data);
 	if (!gs_plugin_download_file (plugin,
 				      app_dl,
@@ -596,7 +641,12 @@ gs_plugin_refresh (GsPlugin *plugin,
 
 		tmp = g_ptr_array_index (priv->to_download, i);
 		basename = g_path_get_basename (tmp);
-		filename_cache = g_build_filename (priv->cachedir, basename, NULL);
+		filename_cache = gs_utils_get_cache_filename ("firmware",
+							      basename,
+							      GS_UTILS_CACHE_FLAG_WRITEABLE,
+							      error);
+		if (filename_cache == NULL)
+			return FALSE;
 		g_debug ("downloading %s to %s", tmp, filename_cache);
 
 		/* set sync request */
@@ -731,13 +781,14 @@ gs_plugin_update_app (GsPlugin *plugin,
  */
 gboolean
 gs_plugin_file_to_app (GsPlugin *plugin,
-		       GList **list,
+		       GsAppList *list,
 		       GFile *file,
 		       GCancellable *cancellable,
 		       GError **error)
 {
 	GsPluginData *priv = gs_plugin_get_data (plugin);
 	g_autofree gchar *content_type = NULL;
+	g_autofree gchar *filename = NULL;
 	g_autoptr(FwupdResult) res = NULL;
 	g_autoptr(GsApp) app = NULL;
 	const gchar *mimetypes[] = {
@@ -752,8 +803,9 @@ gs_plugin_file_to_app (GsPlugin *plugin,
 		return TRUE;
 
 	/* get results */
+	filename = g_file_get_path (file);
 	res = fwupd_client_get_details (priv->client,
-					g_file_get_path (file),
+					filename,
 					cancellable,
 					error);
 	if (res == NULL)
