@@ -20,8 +20,9 @@
  */
 
 #include <config.h>
+
 #include <fnmatch.h>
-#include <gs-plugin.h>
+#include <gnome-software.h>
 
 /*
  * SECTION:
@@ -29,15 +30,13 @@
  */
 
 /**
- * gs_plugin_order_after:
+ * gs_plugin_initialize:
  */
-const gchar **
-gs_plugin_order_after (GsPlugin *plugin)
+void
+gs_plugin_initialize (GsPlugin *plugin)
 {
-	static const gchar *deps[] = {
-		"appstream",		/* need ID */
-		NULL };
-	return deps;
+	/* need ID */
+	gs_plugin_add_rule (plugin, GS_PLUGIN_RULE_RUN_AFTER, "appstream");
 }
 
 /**
