@@ -23,8 +23,7 @@
 #include <config.h>
 
 #include <limba.h>
-#include <appstream-glib.h>
-#include <gs-plugin.h>
+#include <gnome-software.h>
 
 /*
  * SECTION:
@@ -35,18 +34,6 @@ struct GsPluginData {
 	LiManager	*mgr;
 };
 
-/**
- * gs_plugin_order_after:
- */
-const gchar **
-gs_plugin_order_after (GsPlugin *plugin)
-{
-	static const gchar *deps[] = { "appstream",
-				       "packagekit",
-				       NULL };
-	return deps;
-}
-
 /**
  * gs_plugin_initialize:
  */
@@ -55,6 +42,10 @@ gs_plugin_initialize (GsPlugin *plugin)
 {
 	GsPluginData *priv = gs_plugin_alloc_data (plugin, sizeof(GsPluginData));
 	priv->mgr = li_manager_new ();
+
+	/* need help from appstream */
+	gs_plugin_add_rule (plugin, GS_PLUGIN_RULE_RUN_AFTER, "appstream");
+	gs_plugin_add_rule (plugin, GS_PLUGIN_RULE_RUN_AFTER, "packagekit");
 }
 
 /**
@@ -274,7 +265,7 @@ gs_plugin_app_install (GsPlugin *plugin,
  */
 gboolean
 gs_plugin_add_sources (GsPlugin *plugin,
-		       GList **list,
+		       GsAppList *list,
 		       GCancellable *cancellable,
 		       GError **error)
 {
@@ -356,7 +347,7 @@ gs_plugin_app_from_pki (LiPkgInfo *pki)
  */
 gboolean
 gs_plugin_add_installed (GsPlugin *plugin,
-			 GList **list,
+			 GsAppList *list,
 			 GCancellable *cancellable,
 			 GError **error)
 {
@@ -393,7 +384,7 @@ gs_plugin_add_installed (GsPlugin *plugin,
  */
 gboolean
 gs_plugin_add_updates (GsPlugin *plugin,
-			GList **list,
+			GsAppList *list,
 			GCancellable *cancellable,
 			GError **error)
 {
