@@ -21,16 +21,10 @@
 
 #include <config.h>
 
-#include <gs-plugin.h>
+#include <gnome-software.h>
 #include <json-glib/json-glib.h>
 #include <string.h>
 
-#include "gs-app.h"
-#include "gs-os-release.h"
-#include "gs-plugin.h"
-#include "gs-review.h"
-#include "gs-utils.h"
-
 /*
  * SECTION:
  * Provides review data from the Open Desktop Ratings Serice.
@@ -54,6 +48,7 @@ gs_plugin_initialize (GsPlugin *plugin)
 {
 	GsPluginData *priv = gs_plugin_alloc_data (plugin, sizeof(GsPluginData));
 	g_autoptr(GError) error = NULL;
+	g_autoptr(GsOsRelease) os_release = NULL;
 
 	priv->settings = g_settings_new ("org.gnome.software");
 	priv->review_server = g_settings_get_string (priv->settings,
@@ -67,25 +62,21 @@ gs_plugin_initialize (GsPlugin *plugin)
 	}
 
 	/* get the distro name (e.g. 'Fedora') but allow a fallback */
-	priv->distro = gs_os_release_get_name (&error);
+	os_release = gs_os_release_new (&error);
+	if (os_release == NULL) {
+		g_warning ("failed to get distro name: %s", error->message);
+		priv->distro = g_strdup ("Unknown");
+	}
+	priv->distro = g_strdup (gs_os_release_get_name (os_release));
 	if (priv->distro == NULL) {
-		g_warning ("Failed to get distro name: %s", error->message);
+		g_warning ("failed to get distro name");
 		priv->distro = g_strdup ("Unknown");
 	}
-}
 
-
-/**
- * gs_plugin_order_after:
- */
-const gchar **
-gs_plugin_order_after (GsPlugin *plugin)
-{
-	static const gchar *deps[] = {
-		"appstream",	/* need application IDs */
-		"xdg-app",	/* need version */
-		NULL };
-	return deps;
+	/* need application IDs and version */
+	gs_plugin_add_rule (plugin, GS_PLUGIN_RULE_RUN_AFTER, "appstream");
+	gs_plugin_add_rule (plugin, GS_PLUGIN_RULE_RUN_AFTER, "xdg-app");
+	gs_plugin_add_rule (plugin, GS_PLUGIN_RULE_RUN_AFTER, "flatpak");
 }
 
 /**
@@ -403,7 +394,7 @@ gs_plugin_odrs_get_ratings (GsPlugin *plugin, GsApp *app, GError **error)
 	GsPluginData *priv = gs_plugin_get_data (plugin);
 	GArray *ratings;
 	guint status_code;
-	g_autofree gchar *cachedir = NULL;
+	g_autofree gchar *cachefn_basename = NULL;
 	g_autofree gchar *cachefn = NULL;
 	g_autofree gchar *data = NULL;
 	g_autofree gchar *uri = NULL;
@@ -411,10 +402,13 @@ gs_plugin_odrs_get_ratings (GsPlugin *plugin, GsApp *app, GError **error)
 	g_autoptr(SoupMessage) msg = NULL;
 
 	/* look in the cache */
-	cachedir = gs_utils_get_cachedir ("ratings", error);
-	if (cachedir == NULL)
+	cachefn_basename = g_strdup_printf ("%s.json", gs_app_get_id_no_prefix (app));
+	cachefn = gs_utils_get_cache_filename ("ratings",
+					       cachefn_basename,
+					       GS_UTILS_CACHE_FLAG_WRITEABLE,
+					       error);
+	if (cachefn == NULL)
 		return NULL;
-	cachefn = g_strdup_printf ("%s/%s.json", cachedir, gs_app_get_id_no_prefix (app));
 	cachefn_file = g_file_new_for_path (cachefn);
 	if (gs_utils_get_file_age (cachefn_file) < XDG_APP_REVIEW_CACHE_AGE_MAX) {
 		g_autofree gchar *json_data = NULL;
@@ -506,7 +500,7 @@ gs_plugin_odrs_fetch_for_app (GsPlugin *plugin, GsApp *app, GError **error)
 	const gchar *version;
 	guint karma_min;
 	guint status_code;
-	g_autofree gchar *cachedir = NULL;
+	g_autofree gchar *cachefn_basename = NULL;
 	g_autofree gchar *cachefn = NULL;
 	g_autofree gchar *data = NULL;
 	g_autofree gchar *uri = NULL;
@@ -518,10 +512,13 @@ gs_plugin_odrs_fetch_for_app (GsPlugin *plugin, GsApp *app, GError **error)
 	g_autoptr(SoupMessage) msg = NULL;
 
 	/* look in the cache */
-	cachedir = gs_utils_get_cachedir ("reviews", error);
-	if (cachedir == NULL)
+	cachefn_basename = g_strdup_printf ("%s.json", gs_app_get_id_no_prefix (app));
+	cachefn = gs_utils_get_cache_filename ("reviews",
+					       cachefn_basename,
+					       GS_UTILS_CACHE_FLAG_WRITEABLE,
+					       error);
+	if (cachefn == NULL)
 		return NULL;
-	cachefn = g_strdup_printf ("%s/%s.json", cachedir, gs_app_get_id_no_prefix (app));
 	cachefn_file = g_file_new_for_path (cachefn);
 	if (gs_utils_get_file_age (cachefn_file) < XDG_APP_REVIEW_CACHE_AGE_MAX) {
 		g_autofree gchar *json_data = NULL;
@@ -701,17 +698,19 @@ gs_plugin_odrs_sanitize_version (const gchar *version)
 static gboolean
 gs_plugin_odrs_invalidate_cache (GsReview *review, GError **error)
 {
-	g_autofree gchar *cachedir = NULL;
+	g_autofree gchar *cachefn_basename = NULL;
 	g_autofree gchar *cachefn = NULL;
 	g_autoptr(GFile) cachefn_file = NULL;
 
 	/* look in the cache */
-	cachedir = gs_utils_get_cachedir ("reviews", error);
-	if (cachedir == NULL)
+	cachefn_basename = g_strdup_printf ("%s.json",
+					    gs_review_get_metadata_item (review, "app_id"));
+	cachefn = gs_utils_get_cache_filename ("reviews",
+					       cachefn_basename,
+					       GS_UTILS_CACHE_FLAG_WRITEABLE,
+					       error);
+	if (cachefn == NULL)
 		return FALSE;
-	cachefn = g_strdup_printf ("%s/%s.json",
-				   cachedir,
-				   gs_review_get_metadata_item (review, "app_id"));
 	cachefn_file = g_file_new_for_path (cachefn);
 	if (!g_file_query_exists (cachefn_file, NULL))
 		return TRUE;
@@ -949,7 +948,7 @@ gs_plugin_create_app_dummy (const gchar *id)
  */
 gboolean
 gs_plugin_add_unvoted_reviews (GsPlugin *plugin,
-			       GList **list,
+			       GsAppList *list,
 			       GCancellable *cancellable,
 			       GError **error)
 {
