@@ -24,7 +24,7 @@
 #define I_KNOW_THE_PACKAGEKIT_GLIB2_API_IS_SUBJECT_TO_CHANGE
 #include <packagekit-glib2/packagekit.h>
 
-#include <gs-plugin.h>
+#include <gnome-software.h>
 
 /*
  * SECTION:
@@ -101,7 +101,7 @@ gs_plugin_packagekit_convert_error (GError **error,
  */
 gboolean
 gs_plugin_add_updates_historical (GsPlugin *plugin,
-				  GList **list,
+				  GsAppList *list,
 				  GCancellable *cancellable,
 				  GError **error)
 {
