@@ -23,10 +23,7 @@
 
 #define I_KNOW_THE_PACKAGEKIT_GLIB2_API_IS_SUBJECT_TO_CHANGE
 #include <packagekit-glib2/packagekit.h>
-#include <glib/gi18n.h>
-
-#include <gs-plugin.h>
-#include <gs-utils.h>
+#include <gnome-software.h>
 
 #include "packagekit-common.h"
 
@@ -40,18 +37,6 @@ struct GsPluginData {
 	PkTask			*task;
 };
 
-/**
- * gs_plugin_get_conflicts:
- */
-const gchar **
-gs_plugin_get_conflicts (GsPlugin *plugin)
-{
-	static const gchar *deps[] = {
-		"dpkg",
-		NULL };
-	return deps;
-}
-
 /**
  * gs_plugin_initialize:
  */
@@ -63,6 +48,9 @@ gs_plugin_initialize (GsPlugin *plugin)
 	pk_task_set_only_download (priv->task, TRUE);
 	pk_client_set_background (PK_CLIENT (priv->task), TRUE);
 	pk_client_set_interactive (PK_CLIENT (priv->task), FALSE);
+
+	/* we can return better results than dpkg directly */
+	gs_plugin_add_rule (plugin, GS_PLUGIN_RULE_CONFLICTS, "dpkg");
 }
 
 /**
