@@ -23,9 +23,8 @@
 
 #define I_KNOW_THE_PACKAGEKIT_GLIB2_API_IS_SUBJECT_TO_CHANGE
 #include <packagekit-glib2/packagekit.h>
-#include <glib/gi18n.h>
 
-#include <gs-plugin.h>
+#include <gnome-software.h>
 
 #include "packagekit-common.h"
 
@@ -103,7 +102,7 @@ gs_plugin_packagekit_progress_cb (PkProgress *progress,
 
 	} else if (type == PK_PROGRESS_TYPE_PERCENTAGE) {
 		gint percentage = pk_progress_get_percentage (progress);
-		if (percentage >= 0 && percentage <= 100)
+		if (data->app != NULL && percentage >= 0 && percentage <= 100)
 			gs_app_set_progress (data->app, percentage);
 	}
 }
@@ -113,7 +112,7 @@ gs_plugin_packagekit_progress_cb (PkProgress *progress,
  */
 gboolean
 gs_plugin_add_installed (GsPlugin *plugin,
-			 GList **list,
+			 GsAppList *list,
 			 GCancellable *cancellable,
 			 GError **error)
 {
@@ -158,7 +157,7 @@ gs_plugin_add_sources_related (GsPlugin *plugin,
 			       GError **error)
 {
 	GsPluginData *priv = gs_plugin_get_data (plugin);
-	GList *l;
+	guint i;
 	GsApp *app;
 	GsApp *app_tmp;
 	PkBitfield filter;
@@ -188,14 +187,14 @@ gs_plugin_add_sources_related (GsPlugin *plugin,
 	if (!gs_plugin_packagekit_results_valid (results, error))
 		return FALSE;
 	ret = gs_plugin_packagekit_add_results (plugin,
-						&installed,
+						installed,
 						results,
 						error);
 	if (!ret)
 		return FALSE;
-	for (l = installed; l != NULL; l = l->next) {
+	for (i = 0; i < gs_app_list_length (installed); i++) {
 		g_auto(GStrv) split = NULL;
-		app = GS_APP (l->data);
+		app = gs_app_list_index (installed, i);
 		split = pk_package_id_split (gs_app_get_source_id_default (app));
 		if (g_str_has_prefix (split[PK_PACKAGE_ID_DATA], "installed:")) {
 			id = split[PK_PACKAGE_ID_DATA] + 10;
@@ -215,7 +214,7 @@ gs_plugin_add_sources_related (GsPlugin *plugin,
  */
 gboolean
 gs_plugin_add_sources (GsPlugin *plugin,
-		       GList **list,
+		       GsAppList *list,
 		       GCancellable *cancellable,
 		       GError **error)
 {
@@ -633,7 +632,7 @@ gs_plugin_app_remove (GsPlugin *plugin,
 gboolean
 gs_plugin_add_search_files (GsPlugin *plugin,
                             gchar **search,
-                            GList **list,
+                            GsAppList *list,
                             GCancellable *cancellable,
                             GError **error)
 {
@@ -670,7 +669,7 @@ gs_plugin_add_search_files (GsPlugin *plugin,
 gboolean
 gs_plugin_add_search_what_provides (GsPlugin *plugin,
                                     gchar **search,
-                                    GList **list,
+                                    GsAppList *list,
                                     GCancellable *cancellable,
                                     GError **error)
 {
