@@ -24,9 +24,7 @@
 #include <errno.h>
 #include <json-glib/json-glib.h>
 
-#include <gs-plugin.h>
-#include <gs-os-release.h>
-#include <gs-utils.h>
+#include <gnome-software.h>
 
 #define SHELL_EXTENSIONS_API_URI 		"https://extensions.gnome.org/"
 
@@ -219,7 +217,7 @@ gs_plugin_shell_extensions_add_app (const gchar *uuid,
 
 	/* add categories */
 	gs_app_add_category (app, "Addons");
-	gs_app_add_category (app, "shell-extensions");
+	gs_app_add_category (app, "ShellExtensions");
 
 	return g_steal_pointer (&app);
 }
@@ -257,7 +255,7 @@ gs_plugin_setup (GsPlugin *plugin, GCancellable *cancellable, GError **error)
  */
 gboolean
 gs_plugin_add_installed (GsPlugin *plugin,
-			 GList **list,
+			 GsAppList *list,
 			 GCancellable *cancellable,
 			 GError **error)
 {
@@ -476,7 +474,7 @@ gs_plugin_shell_extensions_parse_app (GsPlugin *plugin,
 
 	/* required to match categories in gnome-software */
 	as_app_add_category (app, "Addons");
-	as_app_add_category (app, "shell-extensions");
+	as_app_add_category (app, "ShellExtensions");
 
 	/* we have no data :/ */
 	as_app_set_comment (app, NULL, "GNOME Shell Extension");
@@ -586,7 +584,6 @@ gs_plugin_shell_extensions_get_apps (GsPlugin *plugin,
 {
 	GsPluginData *priv = gs_plugin_get_data (plugin);
 	GPtrArray *apps;
-	g_autofree gchar *cachedir = NULL;
 	g_autofree gchar *cachefn = NULL;
 	g_autofree gchar *uri = NULL;
 	g_autoptr(GFile) cachefn_file = NULL;
@@ -594,10 +591,12 @@ gs_plugin_shell_extensions_get_apps (GsPlugin *plugin,
 	g_autoptr(GsApp) dummy = gs_app_new (gs_plugin_get_name (plugin));
 
 	/* look in the cache */
-	cachedir = gs_utils_get_cachedir ("extensions", error);
-	if (cachedir == NULL)
+	cachefn = gs_utils_get_cache_filename ("extensions",
+					       "gnome.json",
+					       GS_UTILS_CACHE_FLAG_WRITEABLE,
+					       error);
+	if (cachefn == NULL)
 		return NULL;
-	cachefn = g_strdup_printf ("%s/gnome.json", cachedir);
 	cachefn_file = g_file_new_for_path (cachefn);
 	if (gs_utils_get_file_age (cachefn_file) < cache_age) {
 		g_autofree gchar *json_data = NULL;
@@ -857,7 +856,7 @@ gs_plugin_launch (GsPlugin *plugin,
  */
 gboolean
 gs_plugin_add_categories (GsPlugin *plugin,
-			  GList **list,
+			  GPtrArray *list,
 			  GCancellable *cancellable,
 			  GError **error)
 {
