@@ -21,23 +21,19 @@
 
 #include <config.h>
 
-#include <gs-plugin.h>
+#include <gnome-software.h>
 #include <string.h>
 
-#include "gs-utils.h"
-
 #define GS_PLUGIN_STEAM_SCREENSHOT_URI	"http://cdn.akamai.steamstatic.com/steam/apps"
 
 /**
- * gs_plugin_order_after:
+ * gs_plugin_initialize:
  */
-const gchar **
-gs_plugin_order_after (GsPlugin *plugin)
+void
+gs_plugin_initialize (GsPlugin *plugin)
 {
-	static const gchar *deps[] = {
-		"appstream",		/* need metadata */
-		NULL };
-	return deps;
+	/* need metadata */
+	gs_plugin_add_rule (plugin, GS_PLUGIN_RULE_RUN_AFTER, "appstream");
 }
 
 typedef enum {
@@ -399,7 +395,6 @@ gs_plugin_steam_download_icon (GsPlugin *plugin,
 {
 	gsize data_len;
 	g_autofree gchar *cache_basename = NULL;
-	g_autofree gchar *cachedir = NULL;
 	g_autofree gchar *cache_fn = NULL;
 	g_autofree gchar *cache_png = NULL;
 	g_autofree gchar *data = NULL;
@@ -408,10 +403,12 @@ gs_plugin_steam_download_icon (GsPlugin *plugin,
 
 	/* download icons from the cdn */
 	cache_basename = g_path_get_basename (uri);
-	cachedir = gs_utils_get_cachedir ("steam", error);
-	if (cachedir == NULL)
+	cache_fn = gs_utils_get_cache_filename ("steam",
+						cache_basename,
+						GS_UTILS_CACHE_FLAG_NONE,
+						error);
+	if (cache_fn == NULL)
 		return FALSE;
-	cache_fn = g_build_filename (cachedir, cache_basename, NULL);
 	if (g_file_test (cache_fn, G_FILE_TEST_EXISTS)) {
 		if (!g_file_get_contents (cache_fn, &data, &data_len, error))
 			return FALSE;
@@ -446,7 +443,12 @@ gs_plugin_steam_download_icon (GsPlugin *plugin,
 
 	/* save to cache */
 	memcpy (cache_basename + 40, ".png\0", 5);
-	cache_png = g_build_filename (cachedir, cache_basename, NULL);
+	cache_png = gs_utils_get_cache_filename ("steam",
+						 cache_basename,
+						 GS_UTILS_CACHE_FLAG_WRITEABLE,
+						 error);
+	if (cache_png == NULL)
+		return FALSE;
 	if (!gdk_pixbuf_save (pb, cache_png, "png", error, NULL))
 		return FALSE;
 
@@ -472,7 +474,6 @@ gs_plugin_steam_update_store_app (GsPlugin *plugin,
 	guint32 gameid;
 	gchar *app_id;
 	g_autofree gchar *cache_basename = NULL;
-	g_autofree gchar *cachedir = NULL;
 	g_autofree gchar *cache_fn = NULL;
 	g_autofree gchar *gameid_str = NULL;
 	g_autofree gchar *html = NULL;
@@ -610,10 +611,12 @@ gs_plugin_steam_update_store_app (GsPlugin *plugin,
 
 	/* download page from the store */
 	cache_basename = g_strdup_printf ("%s.html", gameid_str);
-	cachedir = gs_utils_get_cachedir ("steam", error);
-	if (cachedir == NULL)
+	cache_fn = gs_utils_get_cache_filename ("steam",
+						cache_basename,
+						GS_UTILS_CACHE_FLAG_WRITEABLE,
+						error);
+	if (cache_fn == NULL)
 		return FALSE;
-	cache_fn = g_build_filename (cachedir, cache_basename, NULL);
 	if (!g_file_test (cache_fn, G_FILE_TEST_EXISTS)) {
 		g_autoptr(GsApp) app_dl = gs_app_new (gs_plugin_get_name (plugin));
 		uri = g_strdup_printf ("http://store.steampowered.com/app/%s/", gameid_str);
@@ -980,7 +983,7 @@ gs_plugin_launch (GsPlugin *plugin, GsApp *app,
 gboolean
 gs_plugin_add_search (GsPlugin *plugin,
 		      gchar **values,
-		      GList **list,
+		      GsAppList *list,
 		      GCancellable *cancellable,
 		      GError **error)
 {
