@@ -23,14 +23,9 @@
 
 #include <string.h>
 #include <math.h>
-#include <libsoup/soup.h>
 #include <json-glib/json-glib.h>
 #include <sqlite3.h>
-
-#include <gs-plugin.h>
-#include <gs-utils.h>
-
-#include "gs-os-release.h"
+#include <gnome-software.h>
 
 struct GsPluginData {
 	gchar		*db_path;
@@ -68,25 +63,9 @@ gs_plugin_initialize (GsPlugin *plugin)
 						  "gnome-software",
 						  "ubuntu-reviews.db",
 						  NULL);
-}
 
-const gchar **
-gs_plugin_order_after (GsPlugin *plugin)
-{
-	static const gchar *deps[] = { NULL };
-	return deps;
-}
-
-/**
- * gs_plugin_get_conflicts:
- */
-const gchar **
-gs_plugin_get_conflicts (GsPlugin *plugin)
-{
-	static const gchar *deps[] = {
-		"odrs",
-		NULL };
-	return deps;
+	/* we have more reviews than ORDS */
+	gs_plugin_add_conflict (plugin, "odrs");
 }
 
 void
