@@ -33,11 +33,9 @@
 #include <config.h>
 
 #include <xdg-app.h>
-
-#include <gs-plugin.h>
+#include <gnome-software.h>
 
 #include "gs-appstream.h"
-#include "gs-utils.h"
 
 static gboolean		gs_plugin_refine_item_metadata (GsPlugin *plugin,
 							GsApp *app,
@@ -49,18 +47,6 @@ struct GsPluginData {
 	GFileMonitor		*monitor;
 };
 
-/**
- * gs_plugin_order_after:
- */
-const gchar **
-gs_plugin_order_after (GsPlugin *plugin)
-{
-	static const gchar *deps[] = {
-		"appstream",
-		NULL };
-	return deps;
-}
-
 /**
  * gs_plugin_initialize:
  */
@@ -68,6 +54,9 @@ void
 gs_plugin_initialize (GsPlugin *plugin)
 {
 	gs_plugin_alloc_data (plugin, sizeof(GsPluginData));
+
+	/* getting app properties from appstream is quicker */
+	gs_plugin_add_rule (plugin, GS_PLUGIN_RULE_RUN_AFTER, "appstream");
 }
 
 /**
@@ -141,7 +130,7 @@ gs_app_set_xdgapp_kind (GsApp *app, XdgAppRefKind kind)
  */
 gboolean
 gs_plugin_add_popular (GsPlugin *plugin,
-		       GList **list,
+		       GsAppList *list,
 		       GCancellable *cancellable,
 		       GError **error)
 {
@@ -447,7 +436,7 @@ gs_plugin_xdg_app_progress_cb (const gchar *status,
  */
 gboolean
 gs_plugin_add_installed (GsPlugin *plugin,
-			 GList **list,
+			 GsAppList *list,
 			 GCancellable *cancellable,
 			 GError **error)
 {
@@ -496,7 +485,7 @@ gs_plugin_add_installed (GsPlugin *plugin,
  */
 gboolean
 gs_plugin_add_sources (GsPlugin *plugin,
-		       GList **list,
+		       GsAppList *list,
 		       GCancellable *cancellable,
 		       GError **error)
 {
@@ -541,7 +530,7 @@ gs_plugin_add_sources (GsPlugin *plugin,
  */
 gboolean
 gs_plugin_add_updates (GsPlugin *plugin,
-		       GList **list,
+		       GsAppList *list,
 		       GCancellable *cancellable,
 		       GError **error)
 {
@@ -1365,7 +1354,7 @@ gs_plugin_update_app (GsPlugin *plugin,
  */
 gboolean
 gs_plugin_file_to_app (GsPlugin *plugin,
-		       GList **list,
+		       GsAppList *list,
 		       GFile *file,
 		       GCancellable *cancellable,
 		       GError **error)
