@@ -24,7 +24,7 @@
 #define I_KNOW_THE_PACKAGEKIT_GLIB2_API_IS_SUBJECT_TO_CHANGE
 #include <packagekit-glib2/packagekit.h>
 
-#include <gs-plugin.h>
+#include <gnome-software.h>
 
 #include "packagekit-common.h"
 
@@ -189,7 +189,7 @@ gs_plugin_packagekit_results_valid (PkResults *results, GError **error)
  */
 gboolean
 gs_plugin_packagekit_add_results (GsPlugin *plugin,
-				  GList **list,
+				  GsAppList *list,
 				  PkResults *results,
 				  GError **error)
 {
